@@ -155,7 +155,11 @@ class CompilerOptions(config.base.Configure):
           flags.extend(['-Wno-unused-but-set-variable'])
       elif bopt in ['g']:
         # -g3 causes an as SEGV on OSX
-        flags.extend(['-g','-O0'])
+        if config.setCompilers.Configure.isHIP(compiler, self.log):
+          # HIP can cause buggy code with -O0
+          flags.extend(['-g'])
+        else:
+          flags.extend(['-g','-O0'])
       elif bopt == 'gcov':
         flags.extend(['--coverage','-Og'])
       elif bopt in ['O']:
@@ -224,7 +228,11 @@ class CompilerOptions(config.base.Configure):
     # Generic
     if not len(flags):
       if bopt in ['g']:
-        flags.extend(['-g','-O0'])
+        if config.setCompilers.Configure.isHIP(compiler, self.log):
+          # HIP can cause buggy code with -O0
+          flags.extend(['-g'])
+        else:
+          flags.extend(['-g','-O0'])
       elif bopt in ['O']:
         flags.append('-O')
     if bopt == 'O':
