@@ -412,7 +412,7 @@ Unable to run hostname to check the network')
                        if (MPI_Win_shared_query(win,0,&size,&disp_unit,&baseptr));\n'):
         self.addDefine('HAVE_MPI_PROCESS_SHARED_MEMORY', 1)
         self.support_mpi3_shm = 1
-    if self.checkLink('#include <mpi.h>\n',
+    if not hasattr(self, 'isIntelMPI') and self.checkLink('#include <mpi.h>\n',
                       'MPI_Aint size=128; int disp_unit=8,*baseptr; MPI_Win win;\n\
                        if (MPI_Win_allocate(size,disp_unit,MPI_INFO_NULL,MPI_COMM_WORLD,&baseptr,&win));\n\
                        if (MPI_Win_attach(win,baseptr,size));\n\
@@ -602,6 +602,7 @@ Unable to run hostname to check the network')
           self.addDefine('HAVE_'+MPICHPKG+'_NUMVERSION',mpich_numversion)
           MPI_VER += '  '+MPICHPKG+'_NUMVERSION: '+mpich_numversion
           if mpichpkg == 'mpich': self.mpich_numversion = mpich_numversion
+          if mpichpkg == 'i_mpi': self.isIntelMPI = 1
         except:
           self.logPrint('Unable to parse '+MPICHPKG+' version from header. Probably a buggy preprocessor')
     if MPI_VER:
