@@ -5,15 +5,16 @@ class Configure(config.package.Package):
     config.package.Package.__init__(self, framework)
     self.cython = 0
     self.numpy = 0
+    self.skippackagewithoptions = 1
     return
 
   def __str__(self):
     return ''
 
   def setupHelp(self,help):
-    config.package.Package.setupHelp(self,help)
     import nargs
-    help.addArgument('PYTHON', '-with-python-exec=<executable>',                nargs.Arg(None, None, 'Alternate Python executable to use for mpi4py/petsc4py'))
+    help.addArgument('PETSc', '-with-python-exec=<executable>', nargs.Arg(None, None, 'Alternate Python executable to use for mpi4py/petsc4py'))
+    help.addArgument('PETSc', '-have-numpy=<bool>', nargs.ArgBool(None, None, 'Whether numpy python module is installed (default: autodetect)'))
     return
 
   def configure(self):
@@ -24,12 +25,18 @@ class Configure(config.package.Package):
       import sys
       self.pyexe = sys.executable
     self.addDefine('PYTHON_EXE','"'+self.pyexe+'"')
+
     try:
       output1,err1,ret1  = config.package.Package.executeShellCommand(self.pyexe + ' -c "import Cython"',timeout=60, log = self.log)
       self.cython = 1
     except: pass
-    try:
-      output1,err1,ret1  = config.package.Package.executeShellCommand(self.pyexe + ' -c "import numpy"',timeout=60, log = self.log)
-      self.numpy = 1
-    except: pass
+
+    have_numpy = self.argDB.get('have-numpy', None)
+    if have_numpy is not None:
+      self.numpy = int(have_numpy)
+    else:
+      try:
+        output1,err1,ret1  = config.package.Package.executeShellCommand(self.pyexe + ' -c "import numpy"',timeout=60, log = self.log)
+        self.numpy = 1
+      except: pass
     return
