@@ -15,16 +15,21 @@ if __name__ == '__main__':
     'COPTFLAGS=-g -O',
     'FOPTFLAGS=-g -O',
     'CXXOPTFLAGS=-g -O',
+    # Note: If using nvcc with a host compiler other than the CUDA SDK default for your platform (GCC on Linux, clang
+    # on Mac OS X, MSVC on Windows), you must set -ccbin appropriately in CUDAFLAGS, as in the example for PGI below:
+    # 'CUDAFLAGS=-ccbin pgc++',
     '--with-cuda=1',
     '--with-precision=double',
     '--with-clanguage=c',
     '--download-kokkos',
     '--download-kokkos-kernels',
     '--download-hwloc',
-    '--with-kokkos-cuda-arch=TURING75',
-    # Note: If using nvcc with a host compiler other than the CUDA SDK default for your platform (GCC on Linux, clang
-    # on Mac OS X, MSVC on Windows), you must set -ccbin appropriately in CUDAFLAGS, as in the example for PGI below:
-    # 'CUDAFLAGS=-ccbin pgc++',
+    # need to pin c++14 since nvcc uses gcc 8 (with std=c++14), petsc uses gcc 9 (with
+    # std=c++17), and interleaving this specific compiler and version combination seems to
+    # break linkage of static constexpr member variables
+    '--with-cxx-dialect=c++14',
+    '--download-hypre',
+    '--download-hypre-configure-arguments=--enable-unified-memory',
   ]
 
   configure.petsc_configure(configure_options)
