@@ -13,7 +13,6 @@ configure_options = [
   '--download-mpich=1',
   '--download-fblaslapack=1',
   '--download-hypre=1',
-  '--download-cmake=1',
   '--download-metis=1',
   '--download-parmetis=1',
   '--download-ptscotch=1',
@@ -35,11 +34,12 @@ configure_options = [
   '--download-szlib=1',
   '--download-zstd=1',
   '--download-moab=1',
-  '--download-petsc4py=1',
+  '--with-petsc4py=1',
   '--download-mpi4py=1',
   '--download-saws',
-  '--download-egads',
   '--package-prefix-hash='+petsc_hash_pkgs,
+  '--download-adolc',
+  '--download-colpack',
   ]
 
 if __name__ == '__main__':
