@@ -1,5 +1,6 @@
 #if !defined(__GAMG_IMPL)
 #define __GAMG_IMPL
+#include <petscksp.h>
 #include <petsc/private/pcimpl.h>
 #include <petsc/private/pcmgimpl.h>                    /*I "petscksp.h" I*/
 #include <petscmatcoarsen.h>                           /*I "petscmatcoarsen.h" I*/
@@ -15,22 +16,22 @@ struct _PCGAMGOps {
   PetscErrorCode (*destroy)(PC);
   PetscErrorCode (*view)(PC,PetscViewer);
 };
-#define PETSC_GAMG_MAXLEVELS 30
 /* Private context for the GAMG preconditioner */
 typedef struct gamg_TAG {
   PCGAMGType type;
   PetscInt  Nlevels;
-  PetscInt  setup_count;
   PetscBool repart;
   PetscBool reuse_prol;
   PetscBool use_aggs_in_asm;
   PetscBool use_parallel_coarse_grid_solver;
+  PCGAMGLayoutType layout_type;
+  PetscBool cpu_pin_coarse_grids;
   PetscInt  min_eq_proc;
   PetscInt  coarse_eq_limit;
   PetscReal threshold_scale;
+  PetscReal threshold[PETSC_MG_MAXLEVELS]; /* common quatity to many AMG methods so keep it up here */
+  PetscInt  level_reduction_factors[PETSC_MG_MAXLEVELS];
   PetscInt  current_level; /* stash construction state */
-  PetscReal threshold[PETSC_GAMG_MAXLEVELS]; /* common quatity to many AMG methods so keep it up here */
-
   /* these 4 are all related to the method data and should be in the subctx */
   PetscInt  data_sz;      /* nloc*data_rows*data_cols */
   PetscInt  data_cell_rows;
@@ -41,17 +42,22 @@ typedef struct gamg_TAG {
   PetscReal *orig_data;          /* cache data */
 
   struct _PCGAMGOps *ops;
-  char *gamg_type_name;
+  char      *gamg_type_name;
 
-  void *subctx;
+  void      *subctx;
+
+  char       esteig_type[32];
+  PetscInt   esteig_max_it;
+  PetscInt   use_sa_esteig;
+  PetscReal  emin,emax;
 } PC_GAMG;
 
 PetscErrorCode PCReset_MG(PC);
 
 /* hooks create derivied classes */
-PetscErrorCode  PCCreateGAMG_GEO(PC);
-PetscErrorCode  PCCreateGAMG_AGG(PC);
-PetscErrorCode  PCCreateGAMG_Classical(PC);
+PetscErrorCode PCCreateGAMG_GEO(PC);
+PetscErrorCode PCCreateGAMG_AGG(PC);
+PetscErrorCode PCCreateGAMG_Classical(PC);
 
 PetscErrorCode PCDestroy_GAMG(PC);
 
@@ -60,12 +66,8 @@ PetscErrorCode PCGAMGCreateGraph(Mat, Mat*);
 PetscErrorCode PCGAMGFilterGraph(Mat*, PetscReal, PetscBool);
 PetscErrorCode PCGAMGGetDataWithGhosts(Mat, PetscInt, PetscReal[],PetscInt*, PetscReal **);
 
-#if defined PETSC_USE_LOG
-#define PETSC_GAMG_USE_LOG
 enum tag {SET1,SET2,GRAPH,GRAPH_MAT,GRAPH_FILTER,GRAPH_SQR,SET4,SET5,SET6,FIND_V,SET7,SET8,SET9,SET10,SET11,SET12,SET13,SET14,SET15,SET16,NUM_SET};
-#if defined PETSC_GAMG_USE_LOG
 PETSC_EXTERN PetscLogEvent petsc_gamg_setup_events[NUM_SET];
-#endif
 PETSC_EXTERN PetscLogEvent PC_GAMGGraph_AGG;
 PETSC_EXTERN PetscLogEvent PC_GAMGGraph_GEO;
 PETSC_EXTERN PetscLogEvent PC_GAMGCoarsen_AGG;
@@ -73,7 +75,7 @@ PETSC_EXTERN PetscLogEvent PC_GAMGCoarsen_GEO;
 PETSC_EXTERN PetscLogEvent PC_GAMGProlongator_AGG;
 PETSC_EXTERN PetscLogEvent PC_GAMGProlongator_GEO;
 PETSC_EXTERN PetscLogEvent PC_GAMGOptProlongator_AGG;
-#endif
+PETSC_EXTERN PetscLogEvent petsc_gamg_setup_matmat_events[PETSC_MG_MAXLEVELS][3];
 
 typedef struct _PCGAMGHashTable {
   PetscInt *table;
@@ -81,7 +83,6 @@ typedef struct _PCGAMGHashTable {
   PetscInt size;
 } PCGAMGHashTable;
 
-
 PETSC_INTERN PetscErrorCode PCGAMGHashTableCreate(PetscInt, PCGAMGHashTable*);
 PETSC_INTERN PetscErrorCode PCGAMGHashTableDestroy(PCGAMGHashTable*);
 PETSC_INTERN PetscErrorCode PCGAMGHashTableAdd(PCGAMGHashTable*,PetscInt,PetscInt);
@@ -92,7 +93,7 @@ PETSC_STATIC_INLINE PetscErrorCode PCGAMGHashTableFind(PCGAMGHashTable *a_tab, P
   PetscInt kk,idx;
 
   PetscFunctionBegin;
-  if (a_key<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Negative key %d.",a_key);
+  if (a_key<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Negative key %D.",a_key);
   for (kk = 0, idx = GAMG_HASH(a_key); kk < a_tab->size; kk++, idx = (idx==(a_tab->size-1)) ? 0 : idx + 1) {
     if (a_tab->table[idx] == a_key) {
       *a_data = a_tab->data[idx];
@@ -103,7 +104,7 @@ PETSC_STATIC_INLINE PetscErrorCode PCGAMGHashTableFind(PCGAMGHashTable *a_tab, P
       break;
     }
   }
-  if (kk==a_tab->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"key %d not found in table",a_key);
+  if (kk==a_tab->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"key %D not found in table",a_key);
   PetscFunctionReturn(0);
 }
 
