@@ -1,10 +1,11 @@
 #ifndef __TAO_IMPL_H
 #define __TAO_IMPL_H
 
+#include <petsctao.h>
 #include <petsctaolinesearch.h>
 #include <petsc/private/petscimpl.h>
-#include <petscksp.h>
 
+PETSC_EXTERN PetscBool TaoRegisterAllCalled;
 PETSC_EXTERN PetscErrorCode TaoRegisterAll(void);
 
 typedef struct _TaoOps *TaoOps;
@@ -15,7 +16,8 @@ struct _TaoOps {
     PetscErrorCode (*computeobjectiveandgradient)(Tao, Vec, PetscReal*, Vec, void*);
     PetscErrorCode (*computegradient)(Tao, Vec, Vec, void*);
     PetscErrorCode (*computehessian)(Tao, Vec, Mat, Mat,  void*);
-    PetscErrorCode (*computeseparableobjective)(Tao, Vec, Vec, void*);
+    PetscErrorCode (*computeresidual)(Tao, Vec, Vec, void*);
+    PetscErrorCode (*computeresidualjacobian)(Tao, Vec, Mat, Mat, void*);
     PetscErrorCode (*computeconstraints)(Tao, Vec, Vec, void*);
     PetscErrorCode (*computeinequalityconstraints)(Tao, Vec, Vec, void*);
     PetscErrorCode (*computeequalityconstraints)(Tao, Vec, Vec, void*);
@@ -25,7 +27,7 @@ struct _TaoOps {
     PetscErrorCode (*computejacobianinequality)(Tao, Vec, Mat, Mat,  void*);
     PetscErrorCode (*computejacobianequality)(Tao, Vec, Mat, Mat,  void*);
     PetscErrorCode (*computebounds)(Tao, Vec, Vec, void*);
-
+    PetscErrorCode (*update)(Tao,PetscInt,void*);
     PetscErrorCode (*convergencetest)(Tao,void*);
     PetscErrorCode (*convergencedestroy)(void*);
 
@@ -47,7 +49,8 @@ struct _p_Tao {
     void *user_objgradP;
     void *user_gradP;
     void *user_hessP;
-    void *user_sepobjP;
+    void *user_lsresP;
+    void *user_lsjacP;
     void *user_conP;
     void *user_con_equalityP;
     void *user_con_inequalityP;
@@ -57,6 +60,7 @@ struct _p_Tao {
     void *user_jac_stateP;
     void *user_jac_designP;
     void *user_boundsP;
+    void *user_update;
 
     PetscErrorCode (*monitor[MAXTAOMONITORS])(Tao,void*);
     PetscErrorCode (*monitordestroy[MAXTAOMONITORS])(void**);
@@ -81,12 +85,14 @@ struct _p_Tao {
     Mat hessian_pre;
     Mat gradient_norm;
     Vec gradient_norm_tmp;
-    Vec sep_objective;
-    Vec sep_weights_v;
-    PetscInt sep_weights_n;
-    PetscInt *sep_weights_rows;
-    PetscInt *sep_weights_cols;
-    PetscReal *sep_weights_w;
+    Vec ls_res;
+    Mat ls_jac;
+    Mat ls_jac_pre;
+    Vec res_weights_v;
+    PetscInt res_weights_n;
+    PetscInt *res_weights_rows;
+    PetscInt *res_weights_cols;
+    PetscReal *res_weights_w;
     Vec constraints;
     Vec constraints_equality;
     Vec constraints_inequality;
@@ -110,7 +116,6 @@ struct _p_Tao {
     PetscReal cnorm0;
     PetscReal fc;
 
-
     PetscInt  max_it;
     PetscInt  max_funcs;
     PetscInt  max_constraints;
@@ -132,7 +137,6 @@ struct _p_Tao {
     PetscInt  ksp_its; /* KSP iterations for this solver iteration */
     PetscInt  ksp_tot_its; /* Total (cumulative) KSP iterations */
 
-
     TaoLineSearch linesearch;
     PetscBool lsflag; /* goes up when line search fails */
     KSP ksp;
@@ -162,6 +166,13 @@ struct _p_Tao {
     PetscBool viewconstraints;
     PetscBool viewhessian;
     PetscBool viewjacobian;
+    PetscBool bounded;
+    PetscBool constrained;
+    PetscBool eq_constrained;
+    PetscBool ineq_constrained;
+    PetscBool ineq_doublesided;
+    PetscBool header_printed;
+    PetscBool recycle;
 
     TaoSubsetType subset_type;
     PetscInt      hist_max;/* Number of iteration histories to keep */
@@ -174,7 +185,13 @@ struct _p_Tao {
     PetscBool     hist_malloc;
 };
 
-extern PetscLogEvent Tao_Solve, Tao_ObjectiveEval, Tao_ObjGradientEval, Tao_GradientEval, Tao_HessianEval, Tao_ConstraintsEval, Tao_JacobianEval;
+PETSC_EXTERN PetscLogEvent TAO_Solve;
+PETSC_EXTERN PetscLogEvent TAO_ObjectiveEval;
+PETSC_EXTERN PetscLogEvent TAO_GradientEval;
+PETSC_EXTERN PetscLogEvent TAO_ObjGradEval;
+PETSC_EXTERN PetscLogEvent TAO_HessianEval;
+PETSC_EXTERN PetscLogEvent TAO_ConstraintsEval;
+PETSC_EXTERN PetscLogEvent TAO_JacobianEval;
 
 PETSC_STATIC_INLINE PetscErrorCode TaoLogConvergenceHistory(Tao tao, PetscReal obj, PetscReal resid, PetscReal cnorm, PetscInt totits)
 {
@@ -198,5 +215,8 @@ PETSC_STATIC_INLINE PetscErrorCode TaoLogConvergenceHistory(Tao tao, PetscReal o
 PETSC_INTERN PetscErrorCode TaoVecGetSubVec(Vec, IS, TaoSubsetType, PetscReal, Vec*);
 PETSC_INTERN PetscErrorCode TaoMatGetSubMat(Mat, IS, Vec, TaoSubsetType, Mat*);
 PETSC_INTERN PetscErrorCode TaoGradientNorm(Tao, Vec, NormType, PetscReal*);
+PETSC_INTERN PetscErrorCode TaoEstimateActiveBounds(Vec, Vec, Vec, Vec, Vec, Vec, PetscReal, PetscReal*, IS*, IS*, IS*, IS*, IS*);
+PETSC_INTERN PetscErrorCode TaoBoundStep(Vec, Vec, Vec, IS, IS, IS, PetscReal, Vec);
+PETSC_INTERN PetscErrorCode TaoBoundSolution(Vec, Vec, Vec, PetscReal, PetscInt*, Vec);
 
 #endif
