@@ -10,10 +10,11 @@
 
 #include <petscvec.h>
 #include <petsc/private/petscimpl.h>
-#include <petscviewer.h>
 
 PETSC_EXTERN PetscBool VecRegisterAllCalled;
 PETSC_EXTERN PetscErrorCode VecRegisterAll(void);
+PETSC_EXTERN MPI_Op MPIU_MAXLOC;
+PETSC_EXTERN MPI_Op MPIU_MINLOC;
 
 /* ----------------------------------------------------------------------------*/
 
@@ -101,6 +102,7 @@ struct _VecOps {
   PetscErrorCode (*restorearrayandmemtype)(Vec,PetscScalar**);
   PetscErrorCode (*restorearrayreadandmemtype)(Vec,const PetscScalar**);
   PetscErrorCode (*concatenate)(PetscInt,const Vec[],Vec*,IS*[]);
+  PetscErrorCode (*sum)(Vec,PetscScalar*);
 };
 
 /*
@@ -212,7 +214,6 @@ PETSC_EXTERN PetscErrorCode VecHIPAllocateCheckHost(Vec v);
 PETSC_EXTERN PetscErrorCode VecHIPCopyFromGPU(Vec v);
 #endif
 
-
 /*
      Common header shared by array based vectors,
    currently Vec_Seq and Vec_MPI
