@@ -243,7 +243,7 @@ PetscErrorCode  PetscDualSpaceViewFromOptions(PetscDualSpace A,PetscObject obj,c
 
   Collective on sp
 
-  Input Parameter:
+  Input Parameters:
 + sp - the PetscDualSpace object to view
 - v  - the viewer
 
@@ -274,7 +274,11 @@ PetscErrorCode PetscDualSpaceView(PetscDualSpace sp, PetscViewer v)
 . sp - the PetscDualSpace object to set options for
 
   Options Database:
-. -petscspace_degree the approximation order of the space
++ -petscdualspace_order <order>      - the approximation order of the space
+. -petscdualspace_form_degree <deg>  - the form degree, say 0 for point evaluations, or 2 for area integrals
+. -petscdualspace_components <c>     - the number of components, say d for a vector field
+. -petscdualspace_refdim <d>         - The spatial dimension of the reference cell
+- -petscdualspace_refcell <celltype> - Reference cell type name
 
   Level: intermediate
 
@@ -397,7 +401,6 @@ static PetscErrorCode PetscDualSpaceClearDMData_Internal(PetscDualSpace sp, DM d
   PetscFunctionReturn(0);
 }
 
-
 /*@
   PetscDualSpaceDestroy - Destroys a PetscDualSpace object
 
@@ -773,7 +776,6 @@ PetscErrorCode PetscDualSpaceGetInteriorDimension(PetscDualSpace sp, PetscInt *i
 .  uniform - PETSC_TRUE if (a) the dual space is the same for each point in a stratum of the reference DMPlex, and
              (b) every symmetry of each point in the reference DMPlex is also a symmetry of the point's dual space.
 
-
    Level: advanced
 
    Note: all of the usual spaces on simplex or tensor-product elements will be uniform, only reference cells
@@ -790,7 +792,6 @@ PetscErrorCode PetscDualSpaceGetUniform(PetscDualSpace sp, PetscBool *uniform)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   PetscDualSpaceGetNumDof - Get the number of degrees of freedom for each spatial (topological) dimension
 
@@ -915,7 +916,7 @@ PetscErrorCode PetscDualSpaceSectionSetUp_Internal(PetscDualSpace sp, PetscSecti
     }
   }
   ierr = DMLabelDestroy(&boundary);CHKERRQ(ierr);
-  ierr = PetscSectionSetUp(section);
+  ierr = PetscSectionSetUp(section);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1019,6 +1020,8 @@ PetscErrorCode PetscDualSpacePushForwardSubspaces_Internal(PetscDualSpace sp, Pe
   Output Parameter:
 . refdm - The reference cell
 
+  Note: This DM is on PETSC_COMM_SELF.
+
   Level: intermediate
 
 .seealso: PetscDualSpaceCreate(), DMPLEX
@@ -1028,7 +1031,7 @@ PetscErrorCode PetscDualSpaceCreateReferenceCell(PetscDualSpace sp, PetscInt dim
   PetscErrorCode ierr;
 
   PetscFunctionBeginUser;
-  ierr = DMPlexCreateReferenceCell(PetscObjectComm((PetscObject) sp), dim, simplex, refdm);CHKERRQ(ierr);
+  ierr = DMPlexCreateReferenceCell(PETSC_COMM_SELF, DMPolytopeTypeSimpleShape(dim, simplex), refdm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1159,7 +1162,7 @@ PetscErrorCode PetscDualSpaceApplyDefault(PetscDualSpace sp, PetscInt f, PetscRe
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
-  PetscValidPointer(value, 5);
+  PetscValidPointer(value, 8);
   ierr = PetscDualSpaceGetDM(sp, &dm);CHKERRQ(ierr);
   ierr = PetscDualSpaceGetFunctional(sp, f, &n);CHKERRQ(ierr);
   ierr = PetscQuadratureGetData(n, &dim, &qNc, &Nq, &points, &weights);CHKERRQ(ierr);
@@ -1207,7 +1210,7 @@ PetscErrorCode PetscDualSpaceApplyAllDefault(PetscDualSpace sp, const PetscScala
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
   PetscValidScalarPointer(pointEval, 2);
-  PetscValidScalarPointer(spValue, 5);
+  PetscValidScalarPointer(spValue, 3);
   ierr = PetscDualSpaceGetAllData(sp, NULL, &allMat);CHKERRQ(ierr);
   if (!(sp->allNodeValues)) {
     ierr = MatCreateVecs(allMat, &(sp->allNodeValues), NULL);CHKERRQ(ierr);
@@ -1248,7 +1251,7 @@ PetscErrorCode PetscDualSpaceApplyInteriorDefault(PetscDualSpace sp, const Petsc
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
   PetscValidScalarPointer(pointEval, 2);
-  PetscValidScalarPointer(spValue, 5);
+  PetscValidScalarPointer(spValue, 3);
   ierr = PetscDualSpaceGetInteriorData(sp, NULL, &intMat);CHKERRQ(ierr);
   if (!(sp->intNodeValues)) {
     ierr = MatCreateVecs(intMat, &(sp->intNodeValues), NULL);CHKERRQ(ierr);
@@ -1272,7 +1275,7 @@ PetscErrorCode PetscDualSpaceApplyInteriorDefault(PetscDualSpace sp, const Petsc
   Input Parameter:
 . sp - The dualspace
 
-  Output Parameter:
+  Output Parameters:
 + allNodes - A PetscQuadrature object containing all evaluation nodes
 - allMat - A Mat for the node-to-dof transformation
 
@@ -1309,7 +1312,7 @@ PetscErrorCode PetscDualSpaceGetAllData(PetscDualSpace sp, PetscQuadrature *allN
   Input Parameter:
 . sp - The dualspace
 
-  Output Parameter:
+  Output Parameters:
 + allNodes - A PetscQuadrature object containing all evaluation nodes
 - allMat - A Mat for the node-to-dof transformation
 
@@ -1385,7 +1388,7 @@ PetscErrorCode PetscDualSpaceCreateAllDataDefault(PetscDualSpace sp, PetscQuadra
   Input Parameter:
 . sp - The dualspace
 
-  Output Parameter:
+  Output Parameters:
 + intNodes - A PetscQuadrature object containing all evaluation points needed to evaluate interior degrees of freedom
 - intMat   - A matrix that computes dual space values from point values: size [spdim0 x (npoints * nc)], where spdim0 is
              the size of the constrained layout (PetscSectionGetConstrainStorageSize()) of the dual space section,
@@ -1424,7 +1427,7 @@ PetscErrorCode PetscDualSpaceGetInteriorData(PetscDualSpace sp, PetscQuadrature
   Input Parameter:
 . sp - The dualspace
 
-  Output Parameter:
+  Output Parameters:
 + intNodes - A PetscQuadrature object containing all evaluation points needed to evaluate interior degrees of freedom
 - intMat    - A matrix that computes dual space values from point values: size [spdim0 x (npoints * nc)], where spdim0 is
               the size of the constrained layout (PetscSectionGetConstrainStorageSize()) of the dual space section,
@@ -1555,7 +1558,7 @@ PetscErrorCode PetscDualSpaceApplyFVM(PetscDualSpace sp, PetscInt f, PetscReal t
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
-  PetscValidPointer(value, 5);
+  PetscValidPointer(value, 8);
   ierr = PetscDualSpaceGetDM(sp, &dm);CHKERRQ(ierr);
   ierr = DMGetCoordinateDim(dm, &dimEmbed);CHKERRQ(ierr);
   ierr = PetscDualSpaceGetFunctional(sp, f, &n);CHKERRQ(ierr);
@@ -1605,7 +1608,7 @@ PetscErrorCode PetscDualSpaceGetHeightSubspace(PetscDualSpace sp, PetscInt heigh
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
-  PetscValidPointer(subsp,2);
+  PetscValidPointer(subsp,3);
   if (!(sp->uniform)) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG, "A non-uniform dual space does not have a single dual space at each height");
   *subsp = NULL;
   dm = sp->dm;
@@ -1675,7 +1678,7 @@ PetscErrorCode PetscDualSpaceGetPointSubspace(PetscDualSpace sp, PetscInt point,
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp, PETSCDUALSPACE_CLASSID, 1);
-  PetscValidPointer(bdsp,2);
+  PetscValidPointer(bdsp,3);
   *bdsp = NULL;
   dm = sp->dm;
   ierr = DMPlexGetChart(dm, &pStart, &pEnd);CHKERRQ(ierr);
@@ -1772,7 +1775,7 @@ PetscErrorCode PetscDualSpaceGetFormDegree(PetscDualSpace dsp, PetscInt *k)
   PetscDualSpaceSetFormDegree - Set the form degree k for the k-form the describes the pushforwards/pullbacks of this
   dual space's functionals.
 
-  Input Parameter:
+  Input Parameters:
 + dsp - The PetscDualSpace
 - k   - The *signed* degree k of the k.  If k >= 0, this means that the degrees of freedom are k-forms, and are stored
         in lexicographic order according to the basis of k-forms constructed from the wedge product of 1-forms.  So for example,
