@@ -18,7 +18,7 @@ static PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
 #endif
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)da),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)da),&rank);CHKERRMPI(ierr);
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
@@ -32,7 +32,26 @@ static PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
 
     ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
-    if (format != PETSC_VIEWER_ASCII_VTK && format != PETSC_VIEWER_ASCII_VTK_CELL && format != PETSC_VIEWER_ASCII_GLVIS) {
+    if (format == PETSC_VIEWER_LOAD_BALANCE) {
+      PetscInt      i,nmax = 0,nmin = PETSC_MAX_INT,navg = 0,*nz,nzlocal;
+      DMDALocalInfo info;
+      PetscMPIInt   size;
+      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)da),&size);CHKERRMPI(ierr);
+      ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
+      nzlocal = info.xm*info.ym*info.zm;
+      ierr = PetscMalloc1(size,&nz);CHKERRQ(ierr);
+      ierr = MPI_Allgather(&nzlocal,1,MPIU_INT,nz,1,MPIU_INT,PetscObjectComm((PetscObject)da));CHKERRMPI(ierr);
+      for (i=0; i<(PetscInt)size; i++) {
+        nmax = PetscMax(nmax,nz[i]);
+        nmin = PetscMin(nmin,nz[i]);
+        navg += nz[i];
+      }
+      ierr = PetscFree(nz);CHKERRQ(ierr);
+      navg = navg/size;
+      ierr = PetscViewerASCIIPrintf(viewer,"  Load Balance - Grid Points: Min %D  avg %D  max %D\n",nmin,navg,nmax);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
+    if (format != PETSC_VIEWER_ASCII_VTK_DEPRECATED && format != PETSC_VIEWER_ASCII_VTK_CELL_DEPRECATED && format != PETSC_VIEWER_ASCII_GLVIS) {
       DMDALocalInfo info;
       ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Processor [%d] M %D N %D P %D m %D n %D p %D w %D s %D\n",rank,dd->M,dd->N,dd->P,dd->m,dd->n,dd->p,dd->w,dd->s);CHKERRQ(ierr);
@@ -75,7 +94,7 @@ static PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
 
     ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
     /* first processor draw all node lines */
-    if (!rank) {
+    if (rank == 0) {
       for (k=0; k<dd->P; k++) {
         ymin = 0.0; ymax = (PetscReal)(dd->N - 1);
         for (xmin=(PetscReal)(k*(dd->M+1)); xmin<(PetscReal)(dd->M+(k*(dd->M+1))); xmin++) {
@@ -209,17 +228,15 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   PetscBool        twod;
   PetscErrorCode   ierr;
 
-
   PetscFunctionBegin;
   if (stencil_type == DMDA_STENCIL_BOX && (bx == DM_BOUNDARY_MIRROR || by == DM_BOUNDARY_MIRROR || bz == DM_BOUNDARY_MIRROR)) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Mirror boundary and box stencil");
-  if ((bx == DM_BOUNDARY_MIRROR) || (by == DM_BOUNDARY_MIRROR) || (bz == DM_BOUNDARY_MIRROR)) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Mirror boundary not supported yet in 3d");
   ierr = PetscObjectGetComm((PetscObject) da, &comm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_64BIT_INDICES)
-  if (((PetscInt64) M)*((PetscInt64) N)*((PetscInt64) P)*((PetscInt64) dof) > (PetscInt64) PETSC_MPI_INT_MAX) SETERRQ3(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D (dof) is too large for 32 bit indices",M,N,dof);
+  if (((PetscInt64) M)*((PetscInt64) N)*((PetscInt64) P)*((PetscInt64) dof) > (PetscInt64) PETSC_MPI_INT_MAX) SETERRQ4(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D by %D (dof) is too large for 32 bit indices",M,N,P,dof);
 #endif
 
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRMPI(ierr);
+  ierr = MPI_Comm_rank(comm,&rank);CHKERRMPI(ierr);
 
   if (m != PETSC_DECIDE) {
     if (m < 1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Non-positive number of processors in X direction: %D",m);
@@ -421,7 +438,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   /* determine starting point of each processor */
   nn       = x*y*z;
   ierr     = PetscMalloc2(size+1,&bases,size,&ldims);CHKERRQ(ierr);
-  ierr     = MPI_Allgather(&nn,1,MPIU_INT,ldims,1,MPIU_INT,comm);CHKERRQ(ierr);
+  ierr     = MPI_Allgather(&nn,1,MPIU_INT,ldims,1,MPIU_INT,comm);CHKERRMPI(ierr);
   bases[0] = 0;
   for (i=1; i<=size; i++) bases[i] = ldims[i-1];
   for (i=1; i<=size; i++) bases[i] += bases[i-1];
@@ -433,23 +450,11 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   dd->nlocal = (Xe-Xs)*(Ye-Ys)*(Ze-Zs)*dof;
   ierr       = VecCreateSeqWithArray(PETSC_COMM_SELF,dof,dd->nlocal,NULL,&local);CHKERRQ(ierr);
 
-  /* generate appropriate vector scatters */
-  /* local to global inserts non-ghost point region into global */
-  ierr   = PetscMalloc1((IXe-IXs)*(IYe-IYs)*(IZe-IZs),&idx);CHKERRQ(ierr);
-  left   = xs - Xs; right = left + x;
-  bottom = ys - Ys; top = bottom + y;
-  down   = zs - Zs; up  = down + z;
-  count  = 0;
-  for (i=down; i<up; i++) {
-    for (j=bottom; j<top; j++) {
-      for (k=left; k<right; k++) {
-        idx[count++] = (i*(Ye-Ys) + j)*(Xe-Xs) + k;
-      }
-    }
-  }
+  /* generate global to local vector scatter and local to global mapping*/
 
   /* global to local must include ghost points within the domain,
      but not ghost points outside the domain that aren't periodic */
+  ierr   = PetscMalloc1((IXe-IXs)*(IYe-IYs)*(IZe-IZs),&idx);CHKERRQ(ierr);
   if (stencil_type == DMDA_STENCIL_BOX) {
     left   = IXs - Xs; right = left + (IXe-IXs);
     bottom = IYs - Ys; top = bottom + (IYe-IYs);
@@ -469,7 +474,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
     bottom = ys - Ys; top = bottom + y;
     down   = zs - Zs;   up  = down + z;
     count  = 0;
-    /* the bottom chunck */
+    /* the bottom chunk */
     for (i=(IZs-Zs); i<down; i++) {
       for (j=bottom; j<top; j++) {
         for (k=left; k<right; k++) idx[count++] = (i*(Ye-Ys) + j)*(Xe-Xs) + k;
@@ -809,7 +814,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         if (twod && (s_t < 0)) s_t = bases[n4] + i*x_t + x_t*y_t*z_t - x_t*y_t; /* 2D case */
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (bz == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + x*i + (s_z - k - 1)*x*y;
       }
 
       if (n5 >= 0) { /* directly right */
@@ -867,7 +872,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n10] - (s_y+1-i)*x_t + (k+1)*x_t*y_t;
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       }  else if (by == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (s_y - i)*x;
       }
       if (n11 >= 0) { /* right below */
         x_t = lx[n11 % m];
@@ -886,7 +891,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n12] + (i+1)*x_t - s_x + k*x_t*y_t;
         for (j=0; j<s_x; j++) idx[nn++] = s_t++;
       }  else if (bx == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<s_x; j++) idx[nn++] = 0;
+        for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + s_x - j - 1 + k*x*y + i*x;
       }
 
       /* Interior */
@@ -900,7 +905,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n14] + i*x_t + k*x_t*y_t;
         for (j=0; j<s_x; j++) idx[nn++] = s_t++;
       } else if (bx == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<s_x; j++) idx[nn++] = 0;
+        for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + x - j - 1 + k*x*y + i*x;
       }
     }
 
@@ -919,7 +924,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n16] + (i-1)*x_t + k*x_t*y_t;
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (by == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (y-i)*x;
       }
       if (n17 >= 0) { /* right above */
         x_t = lx[n17 % m];
@@ -978,7 +983,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         if (twod && (s_t >= M*N*P)) s_t = bases[n22] + i*x_t; /* 2d case */
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (bz == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + (z-k-1)*x*y + i*x;
       }
 
       if (n23 >= 0) { /* directly right */
@@ -1089,7 +1094,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (Zs-zs < 0) {
           if (bz == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + x*i + (s_z - k - 1)*x*y;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1157,7 +1162,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (Ys-ys < 0) {
           if (by == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = -1;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (s_y - i)*x;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1182,7 +1187,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<s_x; j++) idx[nn++] = s_t++;
         } else if (Xs-xs < 0) {
           if (bx == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<s_x; j++) idx[nn++] = 0;
+            for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + s_x - j - 1 + k*x*y + i*x;
           } else {
             for (j=0; j<s_x; j++) idx[nn++] = -1;
           }
@@ -1200,7 +1205,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<s_x; j++) idx[nn++] = s_t++;
         } else if (xe-Xe < 0) {
           if (bx == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<s_x; j++) idx[nn++] = 0;
+            for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + x - j - 1 + k*x*y + i*x;
           } else {
             for (j=0; j<s_x; j++) idx[nn++] = -1;
           }
@@ -1225,7 +1230,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (ye-Ye < 0) {
           if (by == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (y-i)*x;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1293,7 +1298,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (ze-Ze < 0) {
           if (bz == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + (z-k-1)*x*y + i*x;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1365,19 +1370,18 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
    DMDACreate3d - Creates an object that will manage the communication of three-dimensional
    regular array data that is distributed across some processors.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - MPI communicator
 .  bx,by,bz - type of ghost nodes the array have.
          Use one of DM_BOUNDARY_NONE, DM_BOUNDARY_GHOSTED, DM_BOUNDARY_PERIODIC.
 .  stencil_type - Type of stencil (DMDA_STENCIL_STAR or DMDA_STENCIL_BOX)
-.  M,N,P - global dimension in each direction of the array 
+.  M,N,P - global dimension in each direction of the array
 .  m,n,p - corresponding number of processors in each dimension
            (or PETSC_DECIDE to have calculated)
 .  dof - number of degrees of freedom per node
@@ -1393,16 +1397,16 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
 
    Options Database Key:
 +  -dm_view - Calls DMView() at the conclusion of DMDACreate3d()
-.  -da_grid_x <nx> - number of grid points in x direction, if M < 0
-.  -da_grid_y <ny> - number of grid points in y direction, if N < 0
-.  -da_grid_z <nz> - number of grid points in z direction, if P < 0
+.  -da_grid_x <nx> - number of grid points in x direction
+.  -da_grid_y <ny> - number of grid points in y direction
+.  -da_grid_z <nz> - number of grid points in z direction
 .  -da_processors_x <MX> - number of processors in x direction
 .  -da_processors_y <MY> - number of processors in y direction
 .  -da_processors_z <MZ> - number of processors in z direction
 .  -da_refine_x <rx> - refinement ratio in x direction
 .  -da_refine_y <ry> - refinement ratio in y direction
 .  -da_refine_z <rz>- refinement ratio in z directio
--  -da_refine <n> - refine the DMDA n times before creating it, , if M, N, or P < 0
+-  -da_refine <n> - refine the DMDA n times before creating it
 
    Level: beginner
 
@@ -1420,11 +1424,10 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
    If you wish to use the options database to change values in the DMDA call DMSetFromOptions() after this call
    but before DMSetUp().
 
-.keywords: distributed array, create, three-dimensional
-
 .seealso: DMDestroy(), DMView(), DMDACreate1d(), DMDACreate2d(), DMGlobalToLocalBegin(), DMDAGetRefinementFactor(),
           DMGlobalToLocalEnd(), DMLocalToGlobalBegin(), DMLocalToLocalBegin(), DMLocalToLocalEnd(), DMDASetRefinementFactor(),
-          DMDAGetInfo(), DMCreateGlobalVector(), DMCreateLocalVector(), DMDACreateNaturalVector(), DMLoad(), DMDAGetOwnershipRanges()
+          DMDAGetInfo(), DMCreateGlobalVector(), DMCreateLocalVector(), DMDACreateNaturalVector(), DMLoad(), DMDAGetOwnershipRanges(),
+          DMStagCreate3d()
 
 @*/
 PetscErrorCode  DMDACreate3d(MPI_Comm comm,DMBoundaryType bx,DMBoundaryType by,DMBoundaryType bz,DMDAStencilType stencil_type,PetscInt M,
