@@ -5,12 +5,10 @@
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToLocalCreate_DA"
 /*
    DMLocalToLocalCreate_DA - Creates the local to local scatter
 
-   Collective on DMDA
+   Collective on da
 
    Input Parameter:
 .  da - the distributed array
@@ -67,14 +65,12 @@ PetscErrorCode  DMLocalToLocalCreate_DA(DM da)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToLocalBegin_DA"
 /*
    DMLocalToLocalBegin_DA - Maps from a local vector (including ghost points
    that contain irrelevant values) to another local vector where the ghost
    points in the second are set correctly. Must be followed by DMLocalToLocalEnd_DA().
 
-   Neighbor-wise Collective on DMDA and Vec
+   Neighbor-wise Collective on da
 
    Input Parameters:
 +  da - the distributed array context
@@ -105,15 +101,13 @@ PetscErrorCode  DMLocalToLocalBegin_DA(DM da,Vec g,InsertMode mode,Vec l)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToLocalEnd_DA"
 /*
    DMLocalToLocalEnd_DA - Maps from a local vector (including ghost points
    that contain irrelevant values) to another local vector where the ghost
-   points in the second are set correctly.  Must be preceeded by
+   points in the second are set correctly.  Must be preceded by
    DMLocalToLocalBegin_DA().
 
-   Neighbor-wise Collective on DMDA and Vec
+   Neighbor-wise Collective on da
 
    Input Parameters:
 +  da - the distributed array context
@@ -138,7 +132,6 @@ PetscErrorCode  DMLocalToLocalEnd_DA(DM da,Vec g,InsertMode mode,Vec l)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(da,DM_CLASSID,1);
   PetscValidHeaderSpecific(g,VEC_CLASSID,2);
-  PetscValidHeaderSpecific(g,VEC_CLASSID,4);
   ierr = VecScatterEnd(dd->ltol,g,l,mode,SCATTER_FORWARD);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
