@@ -39,7 +39,6 @@ PETSC_EXTERN PetscErrorCode DMGlobalToLocalEnd_Moab(DM, Vec, InsertMode, Vec);
 PETSC_EXTERN PetscErrorCode DMLocalToGlobalBegin_Moab(DM, Vec, InsertMode, Vec);
 PETSC_EXTERN PetscErrorCode DMLocalToGlobalEnd_Moab(DM, Vec, InsertMode, Vec);
 
-
 /* Un-implemented routines */
 /*
 PETSC_EXTERN PetscErrorCode DMCreatelocalsection_Moab(DM dm);
@@ -80,11 +79,10 @@ PetscErrorCode DMMoabCreate(MPI_Comm comm, DM *dmb)
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + comm - The communicator for the DMMoab object
 . mbiface - (ptr to) the MOAB Instance; if passed in NULL, MOAB instance is created inside PETSc, and destroyed
          along with the DMMoab
-. pcomm - (ptr to) a ParallelComm; if NULL, creates one internally for the whole communicator
 . ltog_tag - A tag to use to retrieve global id for an entity; if 0, will use GLOBAL_ID_TAG_NAME/tag
 - range - If non-NULL, contains range of entities to which DOFs will be assigned
 
@@ -163,7 +161,6 @@ PetscErrorCode DMMoabCreateMoab(MPI_Comm comm, moab::Interface *mbiface, moab::T
   PetscFunctionReturn(0);
 }
 
-
 #ifdef MOAB_HAVE_MPI
 
 /*@C
@@ -190,13 +187,12 @@ PetscErrorCode DMMoabGetParallelComm(DM dm, moab::ParallelComm **pcomm)
 
 #endif /* MOAB_HAVE_MPI */
 
-
 /*@C
   DMMoabSetInterface - Set the MOAB instance used with this DMMoab
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm      - The DMMoab object being set
 - mbiface - The MOAB instance being set on this DMMoab
 
@@ -218,7 +214,6 @@ PetscErrorCode DMMoabSetInterface(DM dm, moab::Interface *mbiface)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetInterface - Get the MOAB instance used with this DMMoab
 
@@ -245,13 +240,12 @@ PetscErrorCode DMMoabGetInterface(DM dm, moab::Interface **mbiface)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabSetLocalVertices - Set the entities having DOFs on this DMMoab
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm    - The DMMoab object being set
 - range - The entities treated by this DMMoab
 
@@ -289,14 +283,13 @@ PetscErrorCode DMMoabSetLocalVertices(DM dm, moab::Range *range)
   dmmoab->nghost = dmmoab->vghost->size();
 #ifdef MOAB_HAVE_MPI
   PetscErrorCode  ierr;
-  ierr = MPIU_Allreduce(&dmmoab->nloc, &dmmoab->n, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&dmmoab->nloc, &dmmoab->n, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
 #else
   dmmoab->n = dmmoab->nloc;
 #endif
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetAllVertices - Get the entities having DOFs on this DMMoab
 
@@ -319,8 +312,6 @@ PetscErrorCode DMMoabGetAllVertices(DM dm, moab::Range *local)
   PetscFunctionReturn(0);
 }
 
-
-
 /*@C
   DMMoabGetLocalVertices - Get the entities having DOFs on this DMMoab
 
@@ -367,7 +358,6 @@ PetscErrorCode DMMoabGetLocalElements(DM dm, const moab::Range **range)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabSetLocalElements - Set the entities having DOFs on this DMMoab
 
@@ -398,7 +388,7 @@ PetscErrorCode DMMoabSetLocalElements(DM dm, moab::Range *range)
   dmmoab->neleghost = dmmoab->eghost->size();
 #ifdef MOAB_HAVE_MPI
   PetscErrorCode  ierr;
-  ierr = MPIU_Allreduce(&dmmoab->neleloc, &dmmoab->nele, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&dmmoab->neleloc, &dmmoab->nele, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
   PetscInfo2(dm, "Created %D local and %D global elements.\n", dmmoab->neleloc, dmmoab->nele);
 #else
   dmmoab->nele = dmmoab->neleloc;
@@ -406,7 +396,6 @@ PetscErrorCode DMMoabSetLocalElements(DM dm, moab::Range *range)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabSetLocalToGlobalTag - Set the tag used for local to global numbering
 
@@ -427,7 +416,6 @@ PetscErrorCode DMMoabSetLocalToGlobalTag(DM dm, moab::Tag ltogtag)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetLocalToGlobalTag - Get the tag used for local to global numbering
 
@@ -450,13 +438,12 @@ PetscErrorCode DMMoabGetLocalToGlobalTag(DM dm, moab::Tag *ltog_tag)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabSetBlockSize - Set the block size used with this DMMoab
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object being set
 - bs - The block size used with this DMMoab
 
@@ -471,7 +458,6 @@ PetscErrorCode DMMoabSetBlockSize(DM dm, PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetBlockSize - Get the block size used with this DMMoab
 
@@ -494,7 +480,6 @@ PetscErrorCode DMMoabGetBlockSize(DM dm, PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetSize - Get the global vertex size used with this DMMoab
 
@@ -503,7 +488,7 @@ PetscErrorCode DMMoabGetBlockSize(DM dm, PetscInt *bs)
   Input Parameter:
 . dm - The DMMoab object being set
 
-  Output Parameter:
+  Output Parameters:
 + neg - The number of global elements in the DMMoab instance
 - nvg - The number of global vertices in the DMMoab instance
 
@@ -519,7 +504,6 @@ PetscErrorCode DMMoabGetSize(DM dm, PetscInt *neg, PetscInt *nvg)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetLocalSize - Get the local and ghosted vertex size used with this DMMoab
 
@@ -528,7 +512,7 @@ PetscErrorCode DMMoabGetSize(DM dm, PetscInt *neg, PetscInt *nvg)
   Input Parameter:
 . dm - The DMMoab object being set
 
-  Output Parameter:
+  Output Parameters:
 + nel - The number of owned elements in this processor
 . neg - The number of ghosted elements in this processor
 . nvl - The number of owned vertices in this processor
@@ -548,7 +532,6 @@ PetscErrorCode DMMoabGetLocalSize(DM dm, PetscInt *nel, PetscInt *neg, PetscInt
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetOffset - Get the local offset for the global vector
 
@@ -571,7 +554,6 @@ PetscErrorCode DMMoabGetOffset(DM dm, PetscInt *offset)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetDimension - Get the dimension of the DM Mesh
 
@@ -594,7 +576,6 @@ PetscErrorCode DMMoabGetDimension(DM dm, PetscInt *dim)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetHierarchyLevel - Get the current level of the mesh hierarchy
   generated through uniform refinement.
@@ -618,13 +599,12 @@ PetscErrorCode DMMoabGetHierarchyLevel(DM dm, PetscInt *nlevel)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetMaterialBlock - Get the material ID corresponding to the current entity of the DM Mesh
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 - ehandle - The element entity handle
 
@@ -647,13 +627,12 @@ PetscErrorCode DMMoabGetMaterialBlock(DM dm, const moab::EntityHandle ehandle, P
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetVertexCoordinates - Get the coordinates corresponding to the requested vertex entities
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 . nconn - Number of entities whose coordinates are needed
 - conn - The vertex entity handles
@@ -686,17 +665,16 @@ PetscErrorCode DMMoabGetVertexCoordinates(DM dm, PetscInt nconn, const moab::Ent
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetVertexConnectivity - Get the vertex adjacency for the given entity
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 - vhandle - Vertex entity handle
 
-  Output Parameter:
+  Output Parameters:
 + nconn - Number of entities whose coordinates are needed
 - conn - The vertex entity handles
 
@@ -728,13 +706,12 @@ PetscErrorCode DMMoabGetVertexConnectivity(DM dm, moab::EntityHandle vhandle, Pe
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabRestoreVertexConnectivity - Restore the vertex connectivity for the given entity
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 . vhandle - Vertex entity handle
 . nconn - Number of entities whose coordinates are needed
@@ -759,17 +736,16 @@ PetscErrorCode DMMoabRestoreVertexConnectivity(DM dm, moab::EntityHandle ehandle
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetElementConnectivity - Get the vertex adjacency for the given entity
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 - ehandle - Vertex entity handle
 
-  Output Parameter:
+  Output Parameters:
 + nconn - Number of entities whose coordinates are needed
 - conn - The vertex entity handles
 
@@ -797,13 +773,12 @@ PetscErrorCode DMMoabGetElementConnectivity(DM dm, moab::EntityHandle ehandle, P
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabIsEntityOnBoundary - Check whether a given entity is on the boundary (vertex, edge, face, element)
 
   Collective
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 - ent - Entity handle
 
@@ -847,11 +822,10 @@ PetscErrorCode DMMoabIsEntityOnBoundary(DM dm, const moab::EntityHandle ent, Pet
   PetscFunctionReturn(0);
 }
 
-
 /*@C
-  DMMoabIsEntityOnBoundary - Check whether a given entity is on the boundary (vertex, edge, face, element)
+  DMMoabCheckBoundaryVertices - Check whether a given entity is on the boundary (vertex, edge, face, element)
 
-  Input Parameter:
+  Input Parameters:
 + dm - The DMMoab object
 . nconn - Number of handles
 - cnt - Array of entity handles
@@ -880,14 +854,13 @@ PetscErrorCode DMMoabCheckBoundaryVertices(DM dm, PetscInt nconn, const moab::En
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabGetBoundaryMarkers - Return references to the vertices, faces, elements on the boundary
 
   Input Parameter:
 . dm - The DMMoab object
 
-  Output Parameter:
+  Output Parameters:
 + bdvtx - Boundary vertices
 . bdelems - Boundary elements
 - bdfaces - Boundary faces
@@ -910,7 +883,6 @@ PetscErrorCode DMMoabGetBoundaryMarkers(DM dm, const moab::Range **bdvtx, const
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMDestroy_Moab(DM dm)
 {
   PetscErrorCode  ierr;
@@ -967,7 +939,6 @@ PETSC_EXTERN PetscErrorCode DMDestroy_Moab(DM dm)
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMSetFromOptions_Moab(PetscOptionItems *PetscOptionsObject, DM dm)
 {
   PetscErrorCode ierr;
@@ -979,14 +950,13 @@ PETSC_EXTERN PetscErrorCode DMSetFromOptions_Moab(PetscOptionItems *PetscOptions
   ierr  = PetscOptionsBoundedInt("-dm_moab_rw_dbg", "The verbosity level for reading and writing MOAB meshes", "DMView", dmmoab->rw_dbglevel, &dmmoab->rw_dbglevel, NULL,0);CHKERRQ(ierr);
   ierr  = PetscOptionsBool("-dm_moab_partiton_by_rank", "Use partition by rank when reading MOAB meshes from file", "DMView", dmmoab->partition_by_rank, &dmmoab->partition_by_rank, NULL);CHKERRQ(ierr);
   /* TODO: typically, the read options are needed before a DM is completely created and available in which case, the options wont be available ?? */
-  ierr  = PetscOptionsString("-dm_moab_read_opts", "Extra options to enable MOAB reader to load DM from file", "DMView", dmmoab->extra_read_options, dmmoab->extra_read_options, PETSC_MAX_PATH_LEN, NULL);CHKERRQ(ierr);
-  ierr  = PetscOptionsString("-dm_moab_write_opts", "Extra options to enable MOAB writer to serialize DM to file", "DMView", dmmoab->extra_write_options, dmmoab->extra_write_options, PETSC_MAX_PATH_LEN, NULL);CHKERRQ(ierr);
+  ierr  = PetscOptionsString("-dm_moab_read_opts", "Extra options to enable MOAB reader to load DM from file", "DMView", dmmoab->extra_read_options, dmmoab->extra_read_options, sizeof(dmmoab->extra_read_options), NULL);CHKERRQ(ierr);
+  ierr  = PetscOptionsString("-dm_moab_write_opts", "Extra options to enable MOAB writer to serialize DM to file", "DMView", dmmoab->extra_write_options, dmmoab->extra_write_options, sizeof(dmmoab->extra_write_options), NULL);CHKERRQ(ierr);
   ierr  = PetscOptionsEnum("-dm_moab_read_mode", "MOAB parallel read mode", "DMView", MoabReadModes, (PetscEnum)dmmoab->read_mode, (PetscEnum*)&dmmoab->read_mode, NULL);CHKERRQ(ierr);
   ierr  = PetscOptionsEnum("-dm_moab_write_mode", "MOAB parallel write mode", "DMView", MoabWriteModes, (PetscEnum)dmmoab->write_mode, (PetscEnum*)&dmmoab->write_mode, NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMSetUp_Moab(DM dm)
 {
   PetscErrorCode          ierr;
@@ -1031,7 +1001,7 @@ PETSC_EXTERN PetscErrorCode DMSetUp_Moab(DM dm)
     dmmoab->nghost = dmmoab->vghost->size();
 
 #ifdef MOAB_HAVE_MPI
-    ierr = MPIU_Allreduce(&dmmoab->nloc, &dmmoab->n, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(&dmmoab->nloc, &dmmoab->n, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
     PetscInfo4(NULL, "Filset ID: %u, Vertices: local - %D, owned - %D, ghosted - %D.\n", dmmoab->fileset, dmmoab->vlocal->size(), dmmoab->nloc, dmmoab->nghost);
 #else
     dmmoab->n = dmmoab->nloc;
@@ -1067,7 +1037,7 @@ PETSC_EXTERN PetscErrorCode DMSetUp_Moab(DM dm)
     dmmoab->neleghost = dmmoab->eghost->size();
 
 #ifdef MOAB_HAVE_MPI
-    ierr = MPIU_Allreduce(&dmmoab->neleloc, &dmmoab->nele, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(&dmmoab->neleloc, &dmmoab->nele, 1, MPI_INTEGER, MPI_SUM, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
     PetscInfo3(NULL, "%d-dim elements: owned - %D, ghosted - %D.\n", dmmoab->dim, dmmoab->neleloc, dmmoab->neleghost);
 #else
     dmmoab->nele = dmmoab->neleloc;
@@ -1099,8 +1069,8 @@ PETSC_EXTERN PetscErrorCode DMSetUp_Moab(DM dm)
       if (dmmoab->lminmax[1] < dmmoab->gsindices[i]) dmmoab->lminmax[1] = dmmoab->gsindices[i];
     }
 
-    ierr = MPIU_Allreduce(&dmmoab->lminmax[0], &dmmoab->gminmax[0], 1, MPI_INT, MPI_MIN, ((PetscObject)dm)->comm);CHKERRQ(ierr);
-    ierr = MPIU_Allreduce(&dmmoab->lminmax[1], &dmmoab->gminmax[1], 1, MPI_INT, MPI_MAX, ((PetscObject)dm)->comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(&dmmoab->lminmax[0], &dmmoab->gminmax[0], 1, MPI_INT, MPI_MIN, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
+    ierr = MPIU_Allreduce(&dmmoab->lminmax[1], &dmmoab->gminmax[1], 1, MPI_INT, MPI_MAX, ((PetscObject)dm)->comm);CHKERRMPI(ierr);
 
     /* set the GID map */
     for (i = 0; i < totsize; ++i) {
@@ -1262,7 +1232,6 @@ PETSC_EXTERN PetscErrorCode DMSetUp_Moab(DM dm)
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabCreateVertices - Creates and adds several vertices to the primary set represented by the DM.
 
@@ -1301,7 +1270,6 @@ PetscErrorCode DMMoabCreateVertices(DM dm, const PetscReal* coords, PetscInt nve
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabCreateElement - Adds an element of specified type to the primary set represented by the DM.
 
@@ -1340,7 +1308,6 @@ PetscErrorCode DMMoabCreateElement(DM dm, const moab::EntityType type, const moa
   PetscFunctionReturn(0);
 }
 
-
 /*@C
   DMMoabCreateSubmesh - Creates a sub-DM object with a set that contains all vertices/elements of the parent
   in addition to providing support for dynamic mesh modifications. This is useful for AMR calculations to
@@ -1391,7 +1358,6 @@ PetscErrorCode DMMoabCreateSubmesh(DM dm, DM *newdm)
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMMoabView_Ascii(DM dm, PetscViewer viewer)
 {
   DM_Moab          *dmmoab = (DM_Moab*)(dm)->data;
@@ -1402,7 +1368,7 @@ PETSC_EXTERN PetscErrorCode DMMoabView_Ascii(DM dm, PetscViewer viewer)
 
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)dm, &comm);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm, &size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm, &size);CHKERRMPI(ierr);
   ierr = PetscObjectGetName((PetscObject) dm, &name);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
   if (name) {ierr = PetscViewerASCIIPrintf(viewer, "%s in %D dimensions:\n", name, dmmoab->dim);CHKERRQ(ierr);}
@@ -1472,7 +1438,6 @@ PETSC_EXTERN PetscErrorCode DMView_Moab(DM dm, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMInitialize_Moab(DM dm)
 {
   PetscFunctionBegin;
@@ -1507,7 +1472,6 @@ PETSC_EXTERN PetscErrorCode DMInitialize_Moab(DM dm)
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMClone_Moab(DM dm, DM *newdm)
 {
   PetscErrorCode     ierr;
@@ -1522,7 +1486,6 @@ PETSC_EXTERN PetscErrorCode DMClone_Moab(DM dm, DM *newdm)
   PetscFunctionReturn(0);
 }
 
-
 PETSC_EXTERN PetscErrorCode DMCreate_Moab(DM dm)
 {
   PetscErrorCode ierr;
