@@ -6,7 +6,7 @@
     use for solving a linear system with a complex coefficient matrix.
     This option is irrelevant when solving a real system.
 
-    Logically Collective on KSP
+    Logically Collective on ksp
 
     Input Parameters:
 +   ksp - the iterative context
@@ -25,8 +25,6 @@
     Note:
     By default, the matrix is assumed to be complex, Hermitian.
 
-.keywords: CG, conjugate gradient, Hermitian, symmetric, set, type
-
 .seealso: KSP, KSPCG
 @*/
 PetscErrorCode  KSPCGSetType(KSP ksp,KSPCGType type)
@@ -42,7 +40,7 @@ PetscErrorCode  KSPCGSetType(KSP ksp,KSPCGType type)
 /*@
     KSPCGUseSingleReduction - Merge the two inner products needed in CG into a single MPI_Allreduce() call.
 
-    Logically Collective on KSP
+    Logically Collective on ksp
 
     Input Parameters:
 +   ksp - the iterative context
@@ -54,15 +52,13 @@ PetscErrorCode  KSPCGSetType(KSP ksp,KSPCGType type)
     Level: intermediate
 
      The algorithm used in this case is described as Method 1 in Lapack Working Note 56, "Conjugate Gradient Algorithms with Reduced Synchronization Overhead
-     Distributed Memory Multiprocessors", by E. F. D'Azevedo, V. L. Eijkhout, and C. H. Romine, December 3, 1999. V. Eijkhout creates the algorithm
+     Distributed Memory Multiprocessors", by E. F. D'Azevedo, V. L. Eijkhout, and C. H. Romine, December 3, 1999. V. Eijkhout credits the algorithm
      initially to Chronopoulos and Gear.
 
      It requires two extra work vectors than the conventional implementation in PETSc.
 
      See also KSPPIPECG, KSPPIPECR, and KSPGROPPCG that use non-blocking reductions.
 
-.keywords: CG, conjugate gradient, Hermitian, symmetric, set, type
-
 .seealso: KSP, KSPCG, KSPGMRES
 @*/
 PetscErrorCode  KSPCGUseSingleReduction(KSP ksp,PetscBool flg)
@@ -79,7 +75,7 @@ PetscErrorCode  KSPCGUseSingleReduction(KSP ksp,PetscBool flg)
 /*@
     KSPCGSetRadius - Sets the radius of the trust region.
 
-    Logically Collective on KSP
+    Logically Collective on ksp
 
     Input Parameters:
 +   ksp    - the iterative context
@@ -87,8 +83,6 @@ PetscErrorCode  KSPCGUseSingleReduction(KSP ksp,PetscBool flg)
 
     Level: advanced
 
-.keywords:  set, trust region radius
-
 .seealso: KSP, KSPCG, KSPNASH, KSPSTCG, KSPGLTR
 @*/
 PetscErrorCode  KSPCGSetRadius(KSP ksp, PetscReal radius)
@@ -106,7 +100,7 @@ PetscErrorCode  KSPCGSetRadius(KSP ksp, PetscReal radius)
 /*@
     KSPCGGetNormD - Got norm of the direction.
 
-    Collective on KSP
+    Collective on ksp
 
     Input Parameters:
 +   ksp    - the iterative context
@@ -114,8 +108,6 @@ PetscErrorCode  KSPCGSetRadius(KSP ksp, PetscReal radius)
 
     Level: advanced
 
-.keywords:  get, norm direction
-
 .seealso: KSP, KSPCG, KSPNASH, KSPSTCG, KSPGLTR
 @*/
 PetscErrorCode  KSPCGGetNormD(KSP ksp, PetscReal *norm_d)
@@ -131,7 +123,7 @@ PetscErrorCode  KSPCGGetNormD(KSP ksp, PetscReal *norm_d)
 /*@
     KSPCGGetObjFcn - Get objective function value.
 
-    Collective on KSP
+    Collective on ksp
 
     Input Parameters:
 +   ksp   - the iterative context
@@ -139,8 +131,6 @@ PetscErrorCode  KSPCGGetNormD(KSP ksp, PetscReal *norm_d)
 
     Level: advanced
 
-.keywords:  get, objective function
-
 .seealso: KSP, KSPCG, KSPNASH, KSPSTCG, KSPGLTR
 @*/
 PetscErrorCode  KSPCGGetObjFcn(KSP ksp, PetscReal *o_fcn)
