@@ -1,8 +1,6 @@
 
 #include <petsc/private/kspimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_CR"
 static PetscErrorCode KSPSetUp_CR(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -14,8 +12,6 @@ static PetscErrorCode KSPSetUp_CR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_CR"
 static PetscErrorCode  KSPSolve_CR(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -54,12 +50,15 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
     ierr = VecNormBegin(RT,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
     ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);           /*   (RT,ART)           */
     ierr = VecNormEnd  (RT,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
+    KSPCheckNorm(ksp,dp);
   } else if (ksp->normtype == KSP_NORM_NONE) {
-      dp   = 0.0; /* meaningless value that is passed to monitor and convergence test */
+    dp   = 0.0; /* meaningless value that is passed to monitor and convergence test */
+    ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);           /*   (RT,ART)           */
   } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
     ierr = VecNormBegin(R,NORM_2,&dp);CHKERRQ(ierr);         /*   dp <- R'*R         */
     ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);          /*   (RT,ART)           */
     ierr = VecNormEnd  (R,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
+    KSPCheckNorm(ksp,dp);
   } else if (ksp->normtype == KSP_NORM_NATURAL) {
     ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);           /*   (RT,ART)           */
     dp   = PetscSqrtReal(PetscAbsScalar(btop));                  /* dp = sqrt(R,AR)      */
@@ -84,6 +83,7 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
     ierr = KSP_PCApply(ksp,AP,Q);CHKERRQ(ierr);  /*   Q <- B* AP          */
 
     ierr = VecDot(AP,Q,&apq);CHKERRQ(ierr);
+    KSPCheckDot(ksp,apq);
     if (PetscRealPart(apq) <= 0.0) {
       ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
       ierr        = PetscInfo(ksp,"KSPSolve_CR:diverging due to indefinite or negative definite PC\n");CHKERRQ(ierr);
@@ -101,6 +101,7 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
       ierr = VecNormBegin(RT,NORM_2,&dp);CHKERRQ(ierr);      /*   dp <- || RT ||      */
       ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);
       ierr = VecNormEnd  (RT,NORM_2,&dp);CHKERRQ(ierr);      /*   dp <- || RT ||      */
+      KSPCheckNorm(ksp,dp);
     } else if (ksp->normtype == KSP_NORM_NATURAL) {
       ierr = VecDotEnd(RT,ART,&btop);CHKERRQ(ierr);
       dp   = PetscSqrtReal(PetscAbsScalar(btop));                /* dp = sqrt(R,AR)       */
@@ -112,6 +113,7 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
       ierr = VecNormBegin(R,NORM_2,&dp);CHKERRQ(ierr);       /*   dp <- R'*R          */
       ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);
       ierr = VecNormEnd  (R,NORM_2,&dp);CHKERRQ(ierr);       /*   dp <- R'*R          */
+      KSPCheckNorm(ksp,dp);
     } else SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"KSPNormType of %d not supported",(int)ksp->normtype);
     if (PetscAbsScalar(btop) < 0.0) {
       ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
@@ -138,7 +140,6 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-
 /*MC
      KSPCR - This code implements the (preconditioned) conjugate residuals method
 
@@ -147,18 +148,17 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
 
    Level: beginner
 
-   Notes: The operator and the preconditioner must be symmetric for this method. The
+   Notes:
+    The operator and the preconditioner must be symmetric for this method. The
           preconditioner must be POSITIVE-DEFINITE and the operator POSITIVE-SEMIDEFINITE.
           Support only for left preconditioning.
 
    References:
-.   1. - Magnus R. Hestenes and Eduard Stiefel, Methods of Conjugate Gradients for Solving Linear Systems, 
+.   1. - Magnus R. Hestenes and Eduard Stiefel, Methods of Conjugate Gradients for Solving Linear Systems,
    Journal of Research of the National Bureau of Standards Vol. 49, No. 6, December 1952 Research Paper 2379
 
 .seealso: KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPCG
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_CR"
 PETSC_EXTERN PetscErrorCode KSPCreate_CR(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -167,13 +167,14 @@ PETSC_EXTERN PetscErrorCode KSPCreate_CR(KSP ksp)
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NATURAL,PC_LEFT,2);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);
 
   ksp->ops->setup          = KSPSetUp_CR;
   ksp->ops->solve          = KSPSolve_CR;
   ksp->ops->destroy        = KSPDestroyDefault;
   ksp->ops->buildsolution  = KSPBuildSolutionDefault;
   ksp->ops->buildresidual  = KSPBuildResidualDefault;
-  ksp->ops->setfromoptions = 0;
-  ksp->ops->view           = 0;
+  ksp->ops->setfromoptions = NULL;
+  ksp->ops->view           = NULL;
   PetscFunctionReturn(0);
 }
