@@ -39,7 +39,7 @@ static PetscErrorCode KSPAllocateVectors_PIPEGCR(KSP ksp, PetscInt nvecsneeded,
   pipegcr = (KSP_PIPEGCR*)ksp->data;
 
   /* Allocate enough new vectors to add chunksize new vectors, reach nvecsneedtotal, or to reach mmax+1, whichever is smallest */
-  if(pipegcr->nvecs < PetscMin(pipegcr->mmax+1,nvecsneeded)){
+  if (pipegcr->nvecs < PetscMin(pipegcr->mmax+1,nvecsneeded)) {
     nvecsprev = pipegcr->nvecs;
     nnewvecs = PetscMin(PetscMax(nvecsneeded-pipegcr->nvecs,chunksize),pipegcr->mmax+1-pipegcr->nvecs);
     ierr = KSPCreateVecs(ksp,nnewvecs,&pipegcr->ppvecs[pipegcr->nchunks],0,NULL);CHKERRQ(ierr);
@@ -53,7 +53,7 @@ static PetscErrorCode KSPAllocateVectors_PIPEGCR(KSP ksp, PetscInt nvecsneeded,
       ierr = PetscLogObjectParents((PetscObject)ksp,nnewvecs,pipegcr->ptvecs[pipegcr->nchunks]);CHKERRQ(ierr);
     }
     pipegcr->nvecs += nnewvecs;
-    for(i=0;i<nnewvecs;i++){
+    for (i=0;i<nnewvecs;i++) {
       pipegcr->qvecs[nvecsprev+i] = pipegcr->pqvecs[pipegcr->nchunks][i];
       pipegcr->pvecs[nvecsprev+i] = pipegcr->ppvecs[pipegcr->nchunks][i];
       pipegcr->svecs[nvecsprev+i] = pipegcr->psvecs[pipegcr->nchunks][i];
@@ -78,12 +78,9 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
   PetscReal      rnorm=0.0, delta,*eta,*etas;
 
   PetscFunctionBegin;
-
   /* !!PS We have not checked these routines for use with complex numbers. The inner products
      are likely not defined correctly for that case */
-#if (defined(PETSC_USE_COMPLEX) && !defined(PETSC_SKIP_COMPLEX))
-  SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"PIPEGCR has not been implemented for use with complex scalars");
-#endif
+  if (PetscDefined(USE_COMPLEX) && !PetscDefined(SKIP_COMPLEX)) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"PIPEFGMRES has not been implemented for use with complex scalars");
 
   ierr = KSPGetOperators(ksp, &A, &B);CHKERRQ(ierr);
   x = ksp->vec_sol;
@@ -138,7 +135,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     ierr = VecAXPY(x,+alpha,p);CHKERRQ(ierr);
     ierr = VecAXPY(r,-alpha,s);CHKERRQ(ierr);
     ierr = VecAXPY(z,-alpha,q);CHKERRQ(ierr);
-    if(pipegcr->unroll_w){
+    if (pipegcr->unroll_w) {
       ierr = VecAXPY(w,-alpha,t);CHKERRQ(ierr);
     } else {
       ierr = KSP_MatMult(ksp,A,z,w);CHKERRQ(ierr);
@@ -165,7 +162,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     eta = pipegcr->etas+idx;
 
     /* number of old directions to orthogonalize against */
-    switch(pipegcr->truncstrat){
+    switch(pipegcr->truncstrat) {
       case KSP_FCD_TRUNC_TYPE_STANDARD:
         mi = pipegcr->mmax;
         break;
@@ -177,7 +174,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     }
 
     /* Pick old p,s,q,zeta in a way suitable for VecMDot */
-    for(k=PetscMax(0,i-mi),j=0;k<i;j++,k++){
+    for (k=PetscMax(0,i-mi),j=0;k<i;j++,k++) {
       kdx = k % (pipegcr->mmax+1);
       pipegcr->pold[j] = pipegcr->pvecs[kdx];
       pipegcr->sold[j] = pipegcr->svecs[kdx];
@@ -200,7 +197,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     ierr = VecWAXPY(n,-1.0,r,w);CHKERRQ(ierr);              /* m = u + B(w-r): (a) ntmp = w-r              */
     ierr = KSP_PCApply(ksp,n,m);CHKERRQ(ierr);              /* m = u + B(w-r): (b) mtmp = B(ntmp) = B(w-r) */
     ierr = VecAXPY(m,1.0,z);CHKERRQ(ierr);                  /* m = u + B(w-r): (c) m = z + mtmp            */
-    if(pipegcr->unroll_w){
+    if (pipegcr->unroll_w) {
       ierr = KSP_MatMult(ksp,A,m,n);CHKERRQ(ierr);          /* n = Am                                      */
     }
 
@@ -234,12 +231,12 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     ierr = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
     KSPLogResidualHistory(ksp,rnorm);CHKERRQ(ierr);
     ierr = KSPMonitor(ksp,ksp->its,rnorm);CHKERRQ(ierr);
-    ierr = (*ksp->converged)(ksp,ksp->its+1,rnorm,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
-    if (ksp->reason) break;
+    ierr = (*ksp->converged)(ksp,ksp->its,rnorm,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
+    if (ksp->reason) PetscFunctionReturn(0);
 
     /* compute new eta and scale beta */
     *eta = 0.;
-    for(k=PetscMax(0,i-mi),j=0;k<i;j++,k++){
+    for (k=PetscMax(0,i-mi),j=0;k<i;j++,k++) {
       kdx = k % (pipegcr->mmax+1);
       betas[j] /= -etas[kdx];                               /* betak  /= etak */
       *eta -= ((PetscReal)(PetscAbsScalar(betas[j])*PetscAbsScalar(betas[j]))) * etas[kdx];
@@ -248,7 +245,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     *eta += delta;                                          /* etai    = delta -betaik^2 * etak */
 
     /* check breakdown of eta = (s,s) */
-    if(*eta < 0.) {
+    if (*eta < 0.) {
       pipegcr->norm_breakdown = PETSC_TRUE;
       ierr = PetscInfo1(ksp,"Restart due to square root breakdown at it = \n",ksp->its);CHKERRQ(ierr);
       break;
@@ -260,7 +257,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     ierr = VecCopy(z,p);CHKERRQ(ierr);
     ierr = VecCopy(w,s);CHKERRQ(ierr);
     ierr = VecCopy(m,q);CHKERRQ(ierr);
-    if(pipegcr->unroll_w){
+    if (pipegcr->unroll_w) {
       ierr = VecCopy(n,t);CHKERRQ(ierr);
       ierr = VecMAXPY(t,j,betas,pipegcr->told);CHKERRQ(ierr); /* ti <- n  - sum_k beta_k t_k */
     }
@@ -269,6 +266,7 @@ static PetscErrorCode KSPSolve_PIPEGCR_cycle(KSP ksp)
     ierr = VecMAXPY(q,j,betas,pipegcr->qold);CHKERRQ(ierr); /* qi <- m  - sum_k beta_k q_k */
 
   } while (ksp->its < ksp->max_it);
+  if (ksp->its >= ksp->max_it) ksp->reason = KSP_DIVERGED_ITS;
   PetscFunctionReturn(0);
 }
 
@@ -339,7 +337,7 @@ static PetscErrorCode KSPSolve_PIPEGCR(KSP ksp)
 
   do {
     ierr = KSPSolve_PIPEGCR_cycle(ksp);CHKERRQ(ierr);
-    if (ksp->reason) break;
+    if (ksp->reason) PetscFunctionReturn(0);
     if (pipegcr->norm_breakdown) {
       pipegcr->n_restarts++;
       pipegcr->norm_breakdown = PETSC_FALSE;
@@ -361,12 +359,11 @@ static PetscErrorCode KSPView_PIPEGCR(KSP ksp, PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII, &isascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
 
-  if(pipegcr->truncstrat == KSP_FCD_TRUNC_TYPE_STANDARD){
+  if (pipegcr->truncstrat == KSP_FCD_TRUNC_TYPE_STANDARD) {
     truncstr = "Using standard truncation strategy";
-  } else if(pipegcr->truncstrat == KSP_FCD_TRUNC_TYPE_NOTAY){
+  } else if (pipegcr->truncstrat == KSP_FCD_TRUNC_TYPE_NOTAY) {
     truncstr = "Using Notay's truncation strategy";
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Undefined FCD truncation strategy");
-  
 
   if (isascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"  max previous directions = %D\n",pipegcr->mmax);CHKERRQ(ierr);
@@ -374,13 +371,12 @@ static PetscErrorCode KSPView_PIPEGCR(KSP ksp, PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  %s\n",truncstr);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  w unrolling = %D \n", pipegcr->unroll_w);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  restarts performed = %D \n", pipegcr->n_restarts);CHKERRQ(ierr);
-  } else if (isstring) { 
+  } else if (isstring) {
     ierr = PetscViewerStringSPrintf(viewer, "max previous directions = %D, preallocated %D directions, %s truncation strategy", pipegcr->mmax,pipegcr->nprealloc,truncstr);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-
 static PetscErrorCode KSPSetUp_PIPEGCR(KSP ksp)
 {
   KSP_PIPEGCR   *pipegcr = (KSP_PIPEGCR*)ksp->data;
@@ -405,9 +401,9 @@ static PetscErrorCode KSPSetUp_PIPEGCR(KSP ksp)
     ierr = PetscMalloc3(pipegcr->mmax+1,&(pipegcr->tvecs),pipegcr->mmax+1,&(pipegcr->ptvecs),pipegcr->mmax+2,&(pipegcr->told));CHKERRQ(ierr);
   }
   ierr = PetscMalloc4(pipegcr->mmax+2,&(pipegcr->pold),pipegcr->mmax+2,&(pipegcr->sold),pipegcr->mmax+2,&(pipegcr->qold),pipegcr->mmax+2,&(pipegcr->chunksizes));CHKERRQ(ierr);
-  ierr = PetscMalloc3(pipegcr->mmax+2,&(pipegcr->dots),pipegcr->mmax+1,&(pipegcr->etas),pipegcr->mmax+2,&(pipegcr->redux));CHKERRQ(ierr); 
+  ierr = PetscMalloc3(pipegcr->mmax+2,&(pipegcr->dots),pipegcr->mmax+1,&(pipegcr->etas),pipegcr->mmax+2,&(pipegcr->redux));CHKERRQ(ierr);
   /* If the requested number of preallocated vectors is greater than mmax reduce nprealloc */
-  if(pipegcr->nprealloc > pipegcr->mmax+1){
+  if (pipegcr->nprealloc > pipegcr->mmax+1) {
     ierr = PetscInfo2(NULL,"Requested nprealloc=%d is greater than m_max+1=%d. Resetting nprealloc = m_max+1.\n",pipegcr->nprealloc, pipegcr->mmax+1);CHKERRQ(ierr);
   }
 
@@ -422,8 +418,7 @@ static PetscErrorCode KSPSetUp_PIPEGCR(KSP ksp)
     (pipegcr->mmax + 1) *     sizeof(PetscInt) +    /* chunksizes */
     (pipegcr->mmax + 2) *     sizeof(Vec*) +        /* redux */
     (pipegcr->mmax + 2) *     sizeof(PetscScalar) + /* dots */
-    (pipegcr->mmax + 1) *     sizeof(PetscReal)     /* etas */
-  );CHKERRQ(ierr);
+    (pipegcr->mmax + 1) *     sizeof(PetscReal)     /* etas */);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -449,8 +444,8 @@ static PetscErrorCode KSPDestroy_PIPEGCR(KSP ksp)
   VecDestroyVecs(ksp->nwork,&ksp->work); /* Destroy "standard" work vecs */
 
   /* Destroy vectors for old directions and the arrays that manage pointers to them */
-  if(pipegcr->nvecs){
-    for(i=0;i<pipegcr->nchunks;i++){
+  if (pipegcr->nvecs) {
+    for (i=0;i<pipegcr->nchunks;i++) {
       ierr = VecDestroyVecs(pipegcr->chunksizes[i],&pipegcr->ppvecs[i]);CHKERRQ(ierr);
       ierr = VecDestroyVecs(pipegcr->chunksizes[i],&pipegcr->psvecs[i]);CHKERRQ(ierr);
       ierr = VecDestroyVecs(pipegcr->chunksizes[i],&pipegcr->pqvecs[i]);CHKERRQ(ierr);
@@ -553,7 +548,7 @@ PetscErrorCode KSPPIPEGCRSetMmax(KSP ksp,PetscInt mmax)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  PetscValidLogicalCollectiveEnum(ksp,mmax,2);
+  PetscValidLogicalCollectiveInt(ksp,mmax,2);
   pipegcr->mmax=mmax;
   PetscFunctionReturn(0);
 }
@@ -611,7 +606,7 @@ PetscErrorCode KSPPIPEGCRSetNprealloc(KSP ksp,PetscInt nprealloc)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  PetscValidLogicalCollectiveEnum(ksp,nprealloc,2);
+  PetscValidLogicalCollectiveInt(ksp,nprealloc,2);
   pipegcr->nprealloc = nprealloc;
   PetscFunctionReturn(0);
 }
@@ -715,9 +710,9 @@ static PetscErrorCode KSPSetFromOptions_PIPEGCR(PetscOptionItems *PetscOptionsOb
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"KSP PIPEGCR options");CHKERRQ(ierr);
   ierr = PetscOptionsInt("-ksp_pipegcr_mmax","Number of search directions to storue","KSPPIPEGCRSetMmax",pipegcr->mmax,&mmax,&flg);CHKERRQ(ierr);
-  if (flg) ierr = KSPPIPEGCRSetMmax(ksp,mmax);CHKERRQ(ierr);
+  if (flg) {ierr = KSPPIPEGCRSetMmax(ksp,mmax);CHKERRQ(ierr);}
   ierr = PetscOptionsInt("-ksp_pipegcr_nprealloc","Number of directions to preallocate","KSPPIPEGCRSetNprealloc",pipegcr->nprealloc,&nprealloc,&flg);CHKERRQ(ierr);
-  if (flg) { ierr = KSPPIPEGCRSetNprealloc(ksp,nprealloc);CHKERRQ(ierr); }
+  if (flg) {ierr = KSPPIPEGCRSetNprealloc(ksp,nprealloc);CHKERRQ(ierr);}
   ierr = PetscOptionsEnum("-ksp_pipegcr_truncation_type","Truncation approach for directions","KSPFCGSetTruncationType",KSPFCDTruncationTypes,(PetscEnum)pipegcr->truncstrat,(PetscEnum*)&pipegcr->truncstrat,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-ksp_pipegcr_unroll_w","Use unrolling of w","KSPPIPEGCRSetUnrollW",pipegcr->unroll_w,&pipegcr->unroll_w,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
@@ -748,7 +743,7 @@ static PetscErrorCode  KSPPIPEGCRSetModifyPC_PIPEGCR(KSP ksp,KSPPIPEGCRModifyPCF
  Input Parameters:
  +  ksp      - iterative context obtained from KSPCreate()
  .  function - user defined function to modify the preconditioner
- .  ctx      - user provided contex for the modify preconditioner function
+ .  ctx      - user provided context for the modify preconditioner function
  -  destroy  - the function to use to destroy the user provided application context.
 
  Calling Sequence of function:
@@ -785,7 +780,6 @@ PetscErrorCode  KSPPIPEGCRSetModifyPC(KSP ksp,PetscErrorCode (*function)(KSP,Pet
 .   -ksp_pipegcr_nprealloc <N> - the number of vectors to preallocated for storing Krylov directions. Once exhausted new directions are allocated blockwise (default: 5)
 -   -ksp_pipegcr_truncation_type <standard,notay> - which previous search directions to orthogonalize against
 
-
   Notes:
     The PIPEGCR Krylov method supports non-symmetric matrices and permits the use of a preconditioner
     which may vary from one iteration to the next. Users can can define a method to vary the
@@ -810,7 +804,6 @@ PetscErrorCode  KSPPIPEGCRSetModifyPC(KSP ksp,PetscErrorCode (*function)(KSP,Pet
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
            KSPPIPEFGMRES, KSPPIPECG, KSPPIPECR, KSPPIPEFCG,KSPPIPEGCRSetTruncationType(),KSPPIPEGCRSetNprealloc(),KSPPIPEGCRSetUnrollW(),KSPPIPEGCRSetMmax()
 
-
 M*/
 PETSC_EXTERN PetscErrorCode KSPCreate_PIPEGCR(KSP ksp)
 {
