@@ -9,10 +9,10 @@
 #include <../src/ksp/ksp/impls/gmres/gmresimpl.h>
 
 /*@C
-     KSPGMRESModifiedGramSchmidtOrthogonalization -  This is the basic orthogonalization routine 
+     KSPGMRESModifiedGramSchmidtOrthogonalization -  This is the basic orthogonalization routine
                 using modified Gram-Schmidt.
 
-     Collective on KSP
+     Collective on ksp
 
   Input Parameters:
 +   ksp - KSP object, must be associated with GMRES, FGMRES, or LGMRES Krylov method
@@ -21,16 +21,17 @@
    Options Database Keys:
 .  -ksp_gmres_modifiedgramschmidt - Activates KSPGMRESModifiedGramSchmidtOrthogonalization()
 
+   Notes:
+     In general this is much slower than KSPGMRESClassicalGramSchmidtOrthogonalization() but has better stability properties.
+
    Level: intermediate
 
 .seealso:  KSPGMRESSetOrthogonalization(), KSPGMRESClassicalGramSchmidtOrthogonalization(), KSPGMRESGetOrthogonalization()
 
 @*/
-#undef __FUNCT__  
-#define __FUNCT__ "KSPGMRESModifiedGramSchmidtOrthogonalization"
 PetscErrorCode  KSPGMRESModifiedGramSchmidtOrthogonalization(KSP ksp,PetscInt it)
 {
-  KSP_GMRES      *gmres = (KSP_GMRES *)(ksp->data);
+  KSP_GMRES      *gmres = (KSP_GMRES*)(ksp->data);
   PetscErrorCode ierr;
   PetscInt       j;
   PetscScalar    *hh,*hes;
@@ -43,12 +44,13 @@ PetscErrorCode  KSPGMRESModifiedGramSchmidtOrthogonalization(KSP ksp,PetscInt it
   for (j=0; j<=it; j++) {
     /* (vv(it+1), vv(j)) */
     ierr   = VecDot(VEC_VV(it+1),VEC_VV(j),hh);CHKERRQ(ierr);
+    KSPCheckDot(ksp,*hh);
+    if (ksp->reason) break;
     *hes++ = *hh;
     /* vv(it+1) <- vv(it+1) - hh[it+1][j] vv(j) */
-    ierr   = VecAXPY(VEC_VV(it+1),-(*hh++),VEC_VV(j));CHKERRQ(ierr);
+    ierr = VecAXPY(VEC_VV(it+1),-(*hh++),VEC_VV(j));CHKERRQ(ierr);
   }
   ierr = PetscLogEventEnd(KSP_GMRESOrthogonalization,ksp,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
