@@ -1,21 +1,19 @@
 
 #include <petsc/private/kspimpl.h>  /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFGMRESSetModifyPC"
 /*@C
    KSPFGMRESSetModifyPC - Sets the routine used by FGMRES to modify the preconditioner.
 
-   Logically Collective on KSP
+   Logically Collective on ksp
 
    Input Parameters:
 +  ksp - iterative context obtained from KSPCreate
 .  fcn - modifypc function
-.  ctx - optional contex
+.  ctx - optional context
 -  d - optional context destroy routine
 
    Calling Sequence of function:
-    ierr = int fcn(KSP ksp,int total_its,int loc_its,PetscReal res_norm,void*ctx);
+    PetscErrorCode fcn(KSP ksp,PetscInt total_its,PetscInt loc_its,PetscReal res_norm,void*ctx);
 
     ksp - the ksp context being used.
     total_its     - the total number of FGMRES iterations that have occurred.
@@ -49,11 +47,8 @@ PetscErrorCode  KSPFGMRESSetModifyPC(KSP ksp,PetscErrorCode (*fcn)(KSP,PetscInt,
   PetscFunctionReturn(0);
 }
 
-
 /* The following are different routines used to modify the preconditioner */
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFGMRESModifyPCNoChange"
 /*@
 
   KSPFGMRESModifyPCNoChange - this is the default used by fgmres - it doesn't change the preconditioner.
@@ -81,8 +76,6 @@ PetscErrorCode  KSPFGMRESModifyPCNoChange(KSP ksp,PetscInt total_its,PetscInt lo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFGMRESModifyPCKSP"
 /*@
 
  KSPFGMRESModifyPCKSP - modifies the attributes of the
@@ -134,6 +127,3 @@ PetscErrorCode  KSPFGMRESModifyPCKSP(KSP ksp,PetscInt total_its,PetscInt loc_its
   PetscFunctionReturn(0);
 }
 
-
-
-
