@@ -33,7 +33,7 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
   PetscErrorCode ierr;
   PetscInt       it,j,max_k;
   PetscScalar    alfa, beta, num, den, mone;
-  PetscReal      rnorm;
+  PetscReal      rnorm = 0.0;
   Vec            X,B,R,Z;
   KSP_LCD        *lcd;
   Mat            Amat,Pmat;
@@ -62,7 +62,10 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
   }
 
   ierr = KSP_PCApply(ksp,Z,R);CHKERRQ(ierr);                   /*     r <- M^-1z         */
-  ierr = VecNorm(R,NORM_2,&rnorm);CHKERRQ(ierr);
+  if (ksp->normtype != KSP_NORM_NONE) {
+    ierr = VecNorm(R,NORM_2,&rnorm);CHKERRQ(ierr);
+    KSPCheckNorm(ksp,rnorm);
+  }
   ierr = KSPLogResidualHistory(ksp,rnorm);CHKERRQ(ierr);
   ierr       = KSPMonitor(ksp,0,rnorm);CHKERRQ(ierr);
   ksp->rnorm = rnorm;
@@ -82,10 +85,14 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
       ksp->its++;
       ierr = VecDot(lcd->P[it],R,&num);CHKERRQ(ierr);
       ierr = VecDot(lcd->P[it],lcd->Q[it], &den);CHKERRQ(ierr);
+      KSPCheckDot(ksp,den);
       alfa = num/den;
       ierr = VecAXPY(X,alfa,lcd->P[it]);CHKERRQ(ierr);
       ierr = VecAXPY(R,-alfa,lcd->Q[it]);CHKERRQ(ierr);
-      ierr = VecNorm(R,NORM_2,&rnorm);CHKERRQ(ierr);
+      if (ksp->normtype != KSP_NORM_NONE) {
+        ierr = VecNorm(R,NORM_2,&rnorm);CHKERRQ(ierr);
+        KSPCheckNorm(ksp,rnorm);
+      }
 
       ksp->rnorm = rnorm;
       ierr = KSPLogResidualHistory(ksp,rnorm);CHKERRQ(ierr);
@@ -100,6 +107,7 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
 
       for (j = 0; j <= it; j++) {
         ierr = VecDot(lcd->P[j],lcd->Q[it+1],&num);CHKERRQ(ierr);
+        KSPCheckDot(ksp,num);
         ierr = VecDot(lcd->P[j],lcd->Q[j],&den);CHKERRQ(ierr);
         beta = -num/den;
         ierr = VecAXPY(lcd->P[it+1],beta,lcd->P[j]);CHKERRQ(ierr);
@@ -128,7 +136,6 @@ PetscErrorCode KSPReset_LCD(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-
 PetscErrorCode KSPDestroy_LCD(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -191,7 +198,8 @@ PetscErrorCode KSPSetFromOptions_LCD(PetscOptionItems *PetscOptionsObject,KSP ks
 
    Level: beginner
 
-    Notes: Support only for left preconditioning
+    Notes:
+    Support only for left preconditioning
 
     References:
 +    1. - J.Y. Yuan, G.H.Golub, R.J. Plemmons, and W.A.G. Cecilio. Semiconjugate
@@ -212,7 +220,6 @@ PetscErrorCode KSPSetFromOptions_LCD(PetscOptionItems *PetscOptionsObject,KSP ks
 
   Contributed by: Lucia Catabriga <luciac@ices.utexas.edu>
 
-
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
            KSPCGSetType(), KSPLCDSetRestart(), KSPLCDSetHapTol()
 
@@ -226,6 +233,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_LCD(KSP ksp)
   PetscFunctionBegin;
   ierr         = PetscNewLog(ksp,&lcd);CHKERRQ(ierr);
   ksp->data    = (void*)lcd;
+  ierr         = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);
   ierr         = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
   lcd->restart = 30;
   lcd->haptol  = 1.0e-30;
@@ -244,8 +252,3 @@ PETSC_EXTERN PetscErrorCode KSPCreate_LCD(KSP ksp)
   ksp->ops->buildresidual  = KSPBuildResidualDefault;
   PetscFunctionReturn(0);
 }
-
-
-
-
-
