@@ -38,12 +38,12 @@ static PetscErrorCode MatMult_KSP(Mat A,Vec X,Vec Y)
 
     Collective on ksp
 
-    Input Parameter:
+    Input Parameters:
 +   ksp - the Krylov subspace context
 -   mattype - the matrix type to be used
 
     Output Parameter:
-.   mat - the explict preconditioned operator
+.   mat - the explicit preconditioned operator
 
     Notes:
     This computation is done by applying the operators to columns of the
@@ -86,7 +86,7 @@ PetscErrorCode  KSPComputeOperator(KSP ksp, MatType mattype, Mat *mat)
 
    Collective on ksp
 
-   Input Parameter:
+   Input Parameters:
 +  ksp - iterative context obtained from KSPCreate()
 -  n - size of arrays r and c
 
@@ -127,13 +127,13 @@ PetscErrorCode  KSPComputeEigenvaluesExplicitly(KSP ksp,PetscInt nmax,PetscReal
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)ksp,&comm);CHKERRQ(ierr);
   ierr = KSPComputeOperator(ksp,MATDENSE,&BA);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRMPI(ierr);
+  ierr = MPI_Comm_rank(comm,&rank);CHKERRMPI(ierr);
 
   ierr = MatGetSize(BA,&n,&n);CHKERRQ(ierr);
   if (size > 1) { /* assemble matrix on first processor */
     ierr = MatCreate(PetscObjectComm((PetscObject)ksp),&A);CHKERRQ(ierr);
-    if (!rank) {
+    if (rank == 0) {
       ierr = MatSetSizes(A,n,n,n,n);CHKERRQ(ierr);
     } else {
       ierr = MatSetSizes(A,0,0,n,n);CHKERRQ(ierr);
@@ -158,61 +158,8 @@ PetscErrorCode  KSPComputeEigenvaluesExplicitly(KSP ksp,PetscInt nmax,PetscReal
     ierr = MatDenseGetArray(BA,&array);CHKERRQ(ierr);
   }
 
-#if defined(PETSC_HAVE_ESSL)
-  /* ESSL has a different calling sequence for dgeev() and zgeev() than standard LAPACK */
-  if (!rank) {
-    PetscScalar  sdummy,*cwork;
-    PetscReal    *work,*realpart;
-    PetscBLASInt clen,idummy,lwork,bn,zero = 0;
-    PetscInt     *perm;
-
 #if !defined(PETSC_USE_COMPLEX)
-    clen = n;
-#else
-    clen = 2*n;
-#endif
-    ierr   = PetscMalloc1(clen,&cwork);CHKERRQ(ierr);
-    idummy = -1;                /* unused */
-    ierr   = PetscBLASIntCast(n,&bn);CHKERRQ(ierr);
-    lwork  = 5*n;
-    ierr   = PetscMalloc1(lwork,&work);CHKERRQ(ierr);
-    ierr   = PetscMalloc1(n,&realpart);CHKERRQ(ierr);
-    ierr   = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-    PetscStackCallBLAS("LAPACKgeev",LAPACKgeev_(&zero,array,&bn,cwork,&sdummy,&idummy,&idummy,&bn,work,&lwork));
-    ierr = PetscFPTrapPop();CHKERRQ(ierr);
-    ierr = PetscFree(work);CHKERRQ(ierr);
-
-    /* For now we stick with the convention of storing the real and imaginary
-       components of evalues separately.  But is this what we really want? */
-    ierr = PetscMalloc1(n,&perm);CHKERRQ(ierr);
-
-#if !defined(PETSC_USE_COMPLEX)
-    for (i=0; i<n; i++) {
-      realpart[i] = cwork[2*i];
-      perm[i]     = i;
-    }
-    ierr = PetscSortRealWithPermutation(n,realpart,perm);CHKERRQ(ierr);
-    for (i=0; i<n; i++) {
-      r[i] = cwork[2*perm[i]];
-      c[i] = cwork[2*perm[i]+1];
-    }
-#else
-    for (i=0; i<n; i++) {
-      realpart[i] = PetscRealPart(cwork[i]);
-      perm[i]     = i;
-    }
-    ierr = PetscSortRealWithPermutation(n,realpart,perm);CHKERRQ(ierr);
-    for (i=0; i<n; i++) {
-      r[i] = PetscRealPart(cwork[perm[i]]);
-      c[i] = PetscImaginaryPart(cwork[perm[i]]);
-    }
-#endif
-    ierr = PetscFree(perm);CHKERRQ(ierr);
-    ierr = PetscFree(realpart);CHKERRQ(ierr);
-    ierr = PetscFree(cwork);CHKERRQ(ierr);
-  }
-#elif !defined(PETSC_USE_COMPLEX)
-  if (!rank) {
+  if (rank == 0) {
     PetscScalar  *work;
     PetscReal    *realpart,*imagpart;
     PetscBLASInt idummy,lwork;
@@ -246,7 +193,7 @@ PetscErrorCode  KSPComputeEigenvaluesExplicitly(KSP ksp,PetscInt nmax,PetscReal
     ierr = PetscFree2(realpart,imagpart);CHKERRQ(ierr);
   }
 #else
-  if (!rank) {
+  if (rank == 0) {
     PetscScalar  *work,*eigs;
     PetscReal    *rwork;
     PetscBLASInt idummy,lwork;
@@ -321,7 +268,7 @@ PetscErrorCode KSPPlotEigenContours_Private(KSP ksp,PetscInt neig,const PetscRea
   PetscDrawAxis  drawaxis;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ksp),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ksp),&rank);CHKERRMPI(ierr);
   if (rank) PetscFunctionReturn(0);
   M    = 80;
   N    = 80;
@@ -354,7 +301,7 @@ PetscErrorCode KSPPlotEigenContours_Private(KSP ksp,PetscInt neig,const PetscRea
       value[i+j*M] = PetscLogReal(tmod) / PetscLogReal(10.0);
     }
   }
-  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Iteratively Computed Eigen-contours",PETSC_DECIDE,PETSC_DECIDE,450,450,&viewer);CHKERRQ(ierr);
+  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,NULL,"Iteratively Computed Eigen-contours",PETSC_DECIDE,PETSC_DECIDE,450,450,&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscDrawTensorContour(draw,M,N,NULL,NULL,value);CHKERRQ(ierr);
   if (0) {
