@@ -15,6 +15,14 @@ PetscLogEvent KSP_GMRESOrthogonalization, KSP_SetUp, KSP_Solve, KSP_SolveTranspo
 PetscFunctionList KSPList              = NULL;
 PetscBool         KSPRegisterAllCalled = PETSC_FALSE;
 
+/*
+   Contains the list of registered KSP monitors
+*/
+PetscFunctionList KSPMonitorList              = NULL;
+PetscFunctionList KSPMonitorCreateList        = NULL;
+PetscFunctionList KSPMonitorDestroyList       = NULL;
+PetscBool         KSPMonitorRegisterAllCalled = PETSC_FALSE;
+
 /*@C
   KSPLoad - Loads a KSP that has been stored in binary  with KSPView().
 
@@ -81,7 +89,7 @@ PetscErrorCode  KSPLoad(KSP newdm, PetscViewer viewer)
 -  viewer - visualization context
 
    Options Database Keys:
-.  -ksp_view - print the ksp data structure at the end of a KSPSolve call
+.  -ksp_view - print the KSP data structure at the end of a KSPSolve call
 
    Note:
    The available visualization contexts include
@@ -91,9 +99,15 @@ PetscErrorCode  KSPLoad(KSP newdm, PetscViewer viewer)
          the file.  All other processors send their
          data to the first processor to print.
 
+   The available formats include
++     PETSC_VIEWER_DEFAULT - standard output (default)
+-     PETSC_VIEWER_ASCII_INFO_DETAIL - more verbose output for PCBJACOBI and PCASM
+
    The user can open an alternative visualization context with
    PetscViewerASCIIOpen() - output to a specified file.
 
+  In the debugger you can do "call KSPView(ksp,0)" to display the KSP. (The same holds for any PETSc object viewer).
+
    Level: beginner
 
 .seealso: PCView(), PetscViewerASCIIOpen()
@@ -156,8 +170,8 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
     char        type[256];
 
     ierr = PetscObjectGetComm((PetscObject)ksp,&comm);CHKERRQ(ierr);
-    ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
-    if (!rank) {
+    ierr = MPI_Comm_rank(comm,&rank);CHKERRMPI(ierr);
+    if (rank == 0) {
       ierr = PetscViewerBinaryWrite(viewer,&classid,1,PETSC_INT);CHKERRQ(ierr);
       ierr = PetscStrncpy(type,((PetscObject)ksp)->type_name,256);CHKERRQ(ierr);
       ierr = PetscViewerBinaryWrite(viewer,type,256,PETSC_CHAR);CHKERRQ(ierr);
@@ -194,8 +208,8 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
     const char  *name;
 
     ierr = PetscObjectGetName((PetscObject)ksp,&name);CHKERRQ(ierr);
-    ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
-    if (!((PetscObject)ksp)->amsmem && !rank) {
+    ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRMPI(ierr);
+    if (!((PetscObject)ksp)->amsmem && rank == 0) {
       char       dir[1024];
 
       ierr = PetscObjectViewSAWs((PetscObject)ksp,viewer);CHKERRQ(ierr);
@@ -250,7 +264,7 @@ PetscErrorCode  KSPViewFromOptions(KSP A,PetscObject obj,const char name[])
 
    Logically Collective on ksp
 
-   Input Parameter:
+   Input Parameters:
 +  ksp - Krylov solver context
 -  normtype - one of
 $   KSP_NORM_NONE - skips computing the norm, this should generally only be used if you are using
@@ -264,7 +278,6 @@ $                 of the preconditioned residual P^{-1}(b - A x)
 $   KSP_NORM_UNPRECONDITIONED - uses the l2 norm of the true b - Ax residual.
 $   KSP_NORM_NATURAL - supported  by KSPCG, KSPCR, KSPCGNE, KSPCGS
 
-
    Options Database Key:
 .   -ksp_norm_type <none,preconditioned,unpreconditioned,natural>
 
@@ -295,7 +308,7 @@ PetscErrorCode  KSPSetNormType(KSP ksp,KSPNormType normtype)
 
    Logically Collective on ksp
 
-   Input Parameter:
+   Input Parameters:
 +  ksp - Krylov solver context
 -  it  - use -1 to check at all iterations
 
@@ -324,10 +337,9 @@ PetscErrorCode  KSPSetCheckNormIteration(KSP ksp,PetscInt it)
    computing the inner products for the next iteration.  This can reduce communication costs at the expense of doing
    one additional iteration.
 
-
    Logically Collective on ksp
 
-   Input Parameter:
+   Input Parameters:
 +  ksp - Krylov solver context
 -  flg - PETSC_TRUE or PETSC_FALSE
 
@@ -358,7 +370,7 @@ PetscErrorCode  KSPSetLagNorm(KSP ksp,PetscBool flg)
 
    Logically Collective
 
-   Input Arguments:
+   Input Parameters:
 +  ksp - Krylov method
 .  normtype - supported norm type
 .  pcside - preconditioner side that can be used with this norm
@@ -705,8 +717,15 @@ PetscErrorCode  KSPCreate(MPI_Comm comm,KSP *inksp)
   ksp->res_hist_len   = 0;
   ksp->res_hist_max   = 0;
   ksp->res_hist_reset = PETSC_TRUE;
+  ksp->err_hist       = NULL;
+  ksp->err_hist_alloc = NULL;
+  ksp->err_hist_len   = 0;
+  ksp->err_hist_max   = 0;
+  ksp->err_hist_reset = PETSC_TRUE;
   ksp->numbermonitors = 0;
+  ksp->numberreasonviews = 0;
   ksp->setfromoptionscalled = 0;
+  ksp->nmax = PETSC_DECIDE;
 
   ierr                    = KSPConvergedDefaultCreate(&ctx);CHKERRQ(ierr);
   ierr                    = KSPSetConvergenceTest(ksp,KSPConvergedDefault,ctx,KSPConvergedDefaultDestroy);CHKERRQ(ierr);
@@ -856,3 +875,63 @@ PetscErrorCode  KSPRegister(const char sname[],PetscErrorCode (*function)(KSP))
   ierr = PetscFunctionListAdd(&KSPList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+PetscErrorCode KSPMonitorMakeKey_Internal(const char name[], PetscViewerType vtype, PetscViewerFormat format, char key[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscStrncpy(key, name, PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscStrlcat(key, ":", PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscStrlcat(key, vtype, PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscStrlcat(key, ":", PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscStrlcat(key, PetscViewerFormats[format], PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+  KSPMonitorRegister -  Adds Krylov subspace solver monitor routine.
+
+  Not Collective
+
+  Input Parameters:
++ name    - name of a new monitor routine
+. vtype   - A PetscViewerType for the output
+. format  - A PetscViewerFormat for the output
+. monitor - Monitor routine
+. create  - Creation routine, or NULL
+- destroy - Destruction routine, or NULL
+
+  Notes:
+  KSPMonitorRegister() may be called multiple times to add several user-defined monitors.
+
+  Sample usage:
+.vb
+  KSPMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
+.ve
+
+  Then, your monitor can be chosen with the procedural interface via
+$     KSPMonitorSetFromOptions(ksp,"-ksp_monitor_my_monitor","my_monitor",NULL)
+  or at runtime via the option
+$     -ksp_monitor_my_monitor
+
+   Level: advanced
+
+.seealso: KSPMonitorRegisterAll()
+@*/
+PetscErrorCode KSPMonitorRegister(const char name[], PetscViewerType vtype, PetscViewerFormat format,
+                                  PetscErrorCode (*monitor)(KSP, PetscInt, PetscReal, PetscViewerAndFormat *),
+                                  PetscErrorCode (*create)(PetscViewer, PetscViewerFormat, void *, PetscViewerAndFormat **),
+                                  PetscErrorCode (*destroy)(PetscViewerAndFormat **))
+{
+  char           key[PETSC_MAX_PATH_LEN];
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = KSPInitializePackage();CHKERRQ(ierr);
+  ierr = KSPMonitorMakeKey_Internal(name, vtype, format, key);CHKERRQ(ierr);
+  ierr = PetscFunctionListAdd(&KSPMonitorList, key, monitor);CHKERRQ(ierr);
+  if (create)  {ierr = PetscFunctionListAdd(&KSPMonitorCreateList,  key, create);CHKERRQ(ierr);}
+  if (destroy) {ierr = PetscFunctionListAdd(&KSPMonitorDestroyList, key, destroy);CHKERRQ(ierr);}
+  PetscFunctionReturn(0);
+}
