@@ -8,17 +8,15 @@ typedef struct {
   PetscReal      *eigr;
 } KSPMonitor_SAWs;
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSAWsCreate"
 /*@C
    KSPMonitorSAWsCreate - create an SAWs monitor context
 
    Collective
 
-   Input Arguments:
+   Input Parameter:
 .  ksp - KSP to monitor
 
-   Output Arguments:
+   Output Parameter:
 .  ctx - context for monitor
 
    Level: developer
@@ -33,19 +31,17 @@ PetscErrorCode KSPMonitorSAWsCreate(KSP ksp,void **ctx)
   PetscFunctionBegin;
   ierr = PetscNewLog(ksp,&mon);CHKERRQ(ierr);
   mon->viewer = PETSC_VIEWER_SAWS_(PetscObjectComm((PetscObject)ksp));
-  if (!mon->viewer) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_PLIB,"Cannot create SAWs default viewer");CHKERRQ(ierr);
+  if (!mon->viewer) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_PLIB,"Cannot create SAWs default viewer");
   *ctx = (void*)mon;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSAWsDestroy"
 /*@C
    KSPMonitorSAWsDestroy - destroy a monitor context created with KSPMonitorSAWsCreate()
 
    Collective
 
-   Input Arguments:
+   Input Parameter:
 .  ctx - monitor context
 
    Level: developer
@@ -63,12 +59,10 @@ PetscErrorCode KSPMonitorSAWsDestroy(void **ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSAWs"
 /*@C
    KSPMonitorSAWs - monitor solution using SAWs
 
-   Logically Collective on KSP
+   Logically Collective on ksp
 
    Input Parameters:
 +  ksp   - iterative context
@@ -78,8 +72,6 @@ PetscErrorCode KSPMonitorSAWsDestroy(void **ctx)
 
    Level: advanced
 
-.keywords: KSP, CG, monitor, SAWs, singular values
-
 .seealso: KSPMonitorSingularValue(), KSPComputeExtremeSingularValues(), PetscViewerSAWsOpen()
 @*/
 PetscErrorCode KSPMonitorSAWs(KSP ksp,PetscInt n,PetscReal rnorm,void *ctx)
@@ -98,8 +90,8 @@ PetscErrorCode KSPMonitorSAWs(KSP ksp,PetscInt n,PetscReal rnorm,void *ctx)
   if (n) {
     ierr = KSPComputeEigenvalues(ksp,n,mon->eigr,mon->eigi,&mon->neigs);CHKERRQ(ierr);
 
-    ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
-    if (!rank) {
+    ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRMPI(ierr);
+    if (rank == 0) {
       SAWs_Delete("/PETSc/ksp_monitor_saws/eigr");
       SAWs_Delete("/PETSc/ksp_monitor_saws/eigi");
 
