@@ -1,8 +1,8 @@
 
 static char help[] = "Solves a nonlinear system in parallel with a user-defined Newton method.\n\
-Uses KSP to solve the linearized Newton sytems.  This solver\n\
+Uses KSP to solve the linearized Newton systems.  This solver\n\
 is a very simplistic inexact Newton method.  The intent of this code is to\n\
-demonstrate the repeated solution of linear sytems with the same nonzero pattern.\n\
+demonstrate the repeated solution of linear systems with the same nonzero pattern.\n\
 \n\
 This is NOT the recommended approach for solving nonlinear problems with PETSc!\n\
 We urge users to employ the SNES component for solving nonlinear problems whenever\n\
@@ -130,7 +130,7 @@ int main(int argc,char **argv)
   /*
      Create distributed array (DMDA) to manage parallel grid and vectors
   */
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRMPI(ierr);
   Nx   = PETSC_DECIDE; Ny = PETSC_DECIDE;
   ierr = PetscOptionsGetInt(NULL,NULL,"-Nx",&Nx,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetInt(NULL,NULL,"-Ny",&Ny,NULL);CHKERRQ(ierr);
@@ -148,7 +148,6 @@ int main(int argc,char **argv)
   ierr = VecDuplicate(X,&F);CHKERRQ(ierr);
   ierr = VecDuplicate(X,&Y);CHKERRQ(ierr);
 
-
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create matrix data structure for Jacobian
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -188,7 +187,7 @@ int main(int argc,char **argv)
   ierr = ComputeFunction(&user,X,F);CHKERRQ(ierr);   /* Compute F(X)    */
   ierr = VecNorm(F,NORM_2,&fnorm);CHKERRQ(ierr);     /* fnorm = || F || */
   ttol = fnorm*rtol;
-  if (!no_output) PetscPrintf(comm,"Initial function norm = %g\n",(double)fnorm);
+  if (!no_output) {ierr = PetscPrintf(comm,"Initial function norm = %g\n",(double)fnorm);CHKERRQ(ierr);}
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Solve nonlinear system with a user-defined method
@@ -198,7 +197,7 @@ int main(int argc,char **argv)
       This solver is a very simplistic inexact Newton method, with no
       no damping strategies or bells and whistles. The intent of this code
       is  merely to demonstrate the repeated solution with KSP of linear
-      sytems with the same nonzero structure.
+      systems with the same nonzero structure.
 
       This is NOT the recommended approach for solving nonlinear problems
       with PETSc!  We urge users to employ the SNES component for solving
@@ -209,7 +208,7 @@ int main(int argc,char **argv)
   for (i=0; i<max_nonlin_its; i++) {
 
     /*
-        Compute the Jacobian matrix.  
+        Compute the Jacobian matrix.
      */
     ierr = ComputeJacobian(&user,X,J);CHKERRQ(ierr);
 
@@ -426,7 +425,7 @@ PetscErrorCode ComputeFunction(AppCtx *user,Vec X,Vec F)
    Notes:
    Due to grid point reordering with DMDAs, we must always work
    with the local grid points, and then transform them to the new
-   global numbering with the "ltog" mapping 
+   global numbering with the "ltog" mapping
    We cannot work directly with the global numbers for the original
    uniprocessor grid!
 */
