@@ -121,7 +121,7 @@ int main(int argc,char **args)
   /*
      Set exact solution; then compute right-hand-side vector.
      By default we use an exact solution of a vector with all
-     elements of 1.0;  
+     elements of 1.0;
   */
   ierr = VecSet(u,1.0);CHKERRQ(ierr);
   ierr = MatMult(A,u,b);CHKERRQ(ierr);
@@ -214,7 +214,7 @@ int main(int argc,char **args)
 
    test:
       suffix: chebyest_2
-      args: -m 80 -n 80 -ksp_pc_side right -pc_type ksp -ksp_ksp_type chebyshev -ksp_ksp_max_it 5 -ksp_ksp_chebyshev_esteig 0.9,0,0,1.1 -ksp_esteig_ksp_type cg -ksp_monitor_short 
+      args: -m 80 -n 80 -ksp_pc_side right -pc_type ksp -ksp_ksp_type chebyshev -ksp_ksp_max_it 5 -ksp_ksp_chebyshev_esteig 0.9,0,0,1.1 -ksp_esteig_ksp_type cg -ksp_monitor_short
 
    test:
       args: -ksp_monitor_short -m 5 -n 5 -ksp_gmres_cgs_refinement_type refine_always
@@ -316,6 +316,11 @@ int main(int argc,char **args)
       requires: suitesparse
       args: -ksp_type preonly -pc_type lu -pc_factor_mat_solver_type umfpack
 
+   test:
+      suffix: spqr
+      requires: suitesparse
+      args: -ksp_type preonly -pc_type qr -pc_factor_mat_solver_type spqr
+
    test:
      suffix: pc_symmetric
      args: -m 10 -n 9 -ksp_converged_reason -ksp_type gmres -ksp_pc_side symmetric -pc_type cholesky
@@ -327,4 +332,14 @@ int main(int argc,char **args)
    test:
       suffix: pipeprcg_rcw
       args: -ksp_monitor_short -ksp_type pipeprcg -recompute_w false -m 9 -n 9
+
+   test:
+      suffix: pipecg2
+      args: -ksp_monitor_short -ksp_type pipecg2 -m 9 -n 9 -ksp_norm_type {{preconditioned unpreconditioned natural}}
+
+   test:
+      suffix: pipecg2_2
+      nsize: 4
+      args: -ksp_monitor_short -ksp_type pipecg2 -m 15 -n 9 -ksp_norm_type {{preconditioned unpreconditioned natural}}
+
  TEST*/
