@@ -17,8 +17,6 @@ linear solvers on the individual blocks.\n\n";
    Processors: n
 T*/
 
-
-
 /*
   Include "petscksp.h" so that we can use KSP solvers.  Note that this file
   automatically includes:
@@ -47,8 +45,8 @@ int main(int argc,char **args)
 
   ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRMPI(ierr);
+  ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRMPI(ierr);
   n    = m+2;
 
   /* -------------------------------------------------------------------
@@ -75,15 +73,13 @@ int main(int argc,char **args)
   }
   ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatSetOption(A,MAT_SYMMETRIC,PETSC_TRUE);CHKERRQ(ierr);
 
   /*
      Create parallel vectors
   */
-  ierr = VecCreate(PETSC_COMM_WORLD,&u);CHKERRQ(ierr);
-  ierr = VecSetSizes(u,PETSC_DECIDE,m*n);CHKERRQ(ierr);
-  ierr = VecSetFromOptions(u);CHKERRQ(ierr);
-  ierr = VecDuplicate(u,&b);CHKERRQ(ierr);
-  ierr = VecDuplicate(b,&x);CHKERRQ(ierr);
+  ierr = MatCreateVecs(A,&u,&b);CHKERRQ(ierr);
+  ierr = VecDuplicate(u,&x);CHKERRQ(ierr);
 
   /*
      Set exact solution; then compute right-hand-side vector.
@@ -118,7 +114,6 @@ int main(int argc,char **args)
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCSetType(pc,PCBJACOBI);CHKERRQ(ierr);
 
-
   /* -------------------------------------------------------------------
                    Define the problem decomposition
      ------------------------------------------------------------------- */
@@ -138,6 +133,10 @@ int main(int argc,char **args)
   ierr = PCBJacobiSetTotalBlocks(pc,m,blks);CHKERRQ(ierr);
   ierr = PetscFree(blks);CHKERRQ(ierr);
 
+  /*
+    Set runtime options
+  */
+  ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
 
   /* -------------------------------------------------------------------
                Set the linear solvers for the subblocks
@@ -185,7 +184,7 @@ int main(int argc,char **args)
     */
     for (i=0; i<nlocal; i++) {
       ierr = KSPGetPC(subksp[i],&subpc);CHKERRQ(ierr);
-      if (!rank) {
+      if (rank == 0) {
         if (i%2) {
           ierr = PCSetType(subpc,PCILU);CHKERRQ(ierr);
         } else {
@@ -205,11 +204,6 @@ int main(int argc,char **args)
                       Solve the linear system
      ------------------------------------------------------------------- */
 
-  /*
-    Set runtime options
-  */
-  ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
-
   /*
      Solve the linear system
   */
@@ -238,62 +232,77 @@ int main(int argc,char **args)
   return ierr;
 }
 
-
 /*TEST
 
    test:
+      suffix: 1
       nsize: 2
-      args: -ksp_monitor_short -ksp_gmres_cgs_refinement_type refine_always> ex7_1.tmp 2>&1
+      args: -ksp_monitor_short -ksp_gmres_cgs_refinement_type refine_always
 
    test:
       suffix: 2
       nsize: 2
-      args: -ksp_view
+      args: -ksp_view ::ascii_info_detail
 
    test:
       suffix: viennacl
       requires: viennacl
-      args: -ksp_monitor_short -mat_type aijviennacl -vec_type viennacl
+      args: -ksp_monitor_short -mat_type aijviennacl
       output_file: output/ex7_mpiaijcusparse.out
 
    test:
       suffix: viennacl_2
       nsize: 2
       requires: viennacl
-      args: -ksp_monitor_short -mat_type aijviennacl -vec_type viennacl
+      args: -ksp_monitor_short -mat_type aijviennacl
       output_file: output/ex7_mpiaijcusparse_2.out
 
    test:
       suffix: mpiaijcusparse
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -vec_type cuda
+      args: -ksp_monitor_short -mat_type aijcusparse
 
    test:
       suffix: mpiaijcusparse_2
       nsize: 2
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -vec_type cuda
+      args: -ksp_monitor_short -mat_type aijcusparse
 
    test:
       suffix: mpiaijcusparse_simple
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -vec_type cuda -sub_ksp_type preonly -sub_pc_type ilu
+      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -sub_ksp_type preonly -sub_pc_type ilu
 
    test:
       suffix: mpiaijcusparse_simple_2
       nsize: 2
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -vec_type cuda -sub_ksp_type preonly -sub_pc_type ilu
+      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -sub_ksp_type preonly -sub_pc_type ilu
 
    test:
       suffix: mpiaijcusparse_3
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -vec_type cuda
+      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse
 
    test:
       suffix: mpiaijcusparse_4
       nsize: 2
       requires: cuda
-      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse -vec_type cuda
+      args: -ksp_monitor_short -mat_type aijcusparse -sub_pc_factor_mat_solver_type cusparse
+
+   testset:
+      args: -ksp_monitor_short -pc_type gamg -ksp_view
+      test:
+        suffix: gamg_cuda
+        nsize: {{1 2}separate output}
+        requires: cuda
+        args: -mat_type aijcusparse
+        # triggers cusparse MatTransposeMat operation when squaring the graph
+        args: -pc_gamg_sym_graph 0 -pc_gamg_threshold -1 -pc_gamg_square_graph 1
+      test:
+        suffix: gamg_kokkos
+        nsize: {{1 2}separate output}
+        requires: kokkos_kernels
+        args: -mat_type aijkokkos
 
 TEST*/
