@@ -2,11 +2,11 @@
 
 /*@
    MatLMVMUpdate - Adds (X-Xprev) and (F-Fprev) updates to an LMVM matrix.
-   The first time the function is called for an LMVM matrix, no update is 
+   The first time the function is called for an LMVM matrix, no update is
    applied, but the given X and F vectors are stored for use as Xprev and
    Fprev in the next update.
-   
-   If the user has provided another LMVM matrix in place of J0, the J0 
+
+   If the user has provided another LMVM matrix in place of J0, the J0
    matrix is also updated recursively.
 
    Input Parameters:
@@ -49,7 +49,7 @@ PetscErrorCode MatLMVMUpdate(Mat B, Vec X, Vec F)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMClearJ0 - Removes all definitions of J0 and reverts to 
+   MatLMVMClearJ0 - Removes all definitions of J0 and reverts to
    an identity matrix (scale = 1.0).
 
    Input Parameters:
@@ -115,7 +115,7 @@ PetscErrorCode MatLMVMSetJ0Scale(Mat B, PetscReal scale)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMSetJ0Diag - Allows the user to define a vector 
+   MatLMVMSetJ0Diag - Allows the user to define a vector
    V such that J0 = diag(V).
 
    Input Parameters:
@@ -153,17 +153,17 @@ PetscErrorCode MatLMVMSetJ0Diag(Mat B, Vec V)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMSetJ0 - Allows the user to define the initial 
-   Jacobian matrix from which the LMVM approximation is 
-   built up. Inverse of this initial Jacobian is applied 
+   MatLMVMSetJ0 - Allows the user to define the initial
+   Jacobian matrix from which the LMVM approximation is
+   built up. Inverse of this initial Jacobian is applied
    using an internal KSP solver, which defaults to GMRES.
-   This internal KSP solver has the "mat_lmvm_" option 
+   This internal KSP solver has the "mat_lmvm_" option
    prefix.
-   
-   Note that another LMVM matrix can be used in place of 
-   J0, in which case updating the outer LMVM matrix will 
-   also trigger the update for the inner LMVM matrix. This 
-   is useful in cases where a full-memory diagonal approximation 
+
+   Note that another LMVM matrix can be used in place of
+   J0, in which case updating the outer LMVM matrix will
+   also trigger the update for the inner LMVM matrix. This
+   is useful in cases where a full-memory diagonal approximation
    such as MATLMVMDIAGBRDN is used in place of J0.
 
    Input Parameters:
@@ -200,10 +200,10 @@ PetscErrorCode MatLMVMSetJ0(Mat B, Mat J0)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMSetJ0PC - Allows the user to define a PC object that 
-   acts as the initial inverse-Jacobian matrix. This PC should 
-   already contain all the operators necessary for its application. 
-   The LMVM matrix only calls PCApply() without changing any other 
+   MatLMVMSetJ0PC - Allows the user to define a PC object that
+   acts as the initial inverse-Jacobian matrix. This PC should
+   already contain all the operators necessary for its application.
+   The LMVM matrix only calls PCApply() without changing any other
    options.
 
    Input Parameters:
@@ -237,10 +237,10 @@ PetscErrorCode MatLMVMSetJ0PC(Mat B, PC J0pc)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMSetJ0KSP - Allows the user to provide a pre-configured 
-   KSP solver for the initial inverse-Jacobian approximation. 
-   This KSP solver should already contain all the operators 
-   necessary to perform the inversion. The LMVM matrix only 
+   MatLMVMSetJ0KSP - Allows the user to provide a pre-configured
+   KSP solver for the initial inverse-Jacobian approximation.
+   This KSP solver should already contain all the operators
+   necessary to perform the inversion. The LMVM matrix only
    calls KSPSolve() without changing any other options.
 
    Input Parameters:
@@ -304,7 +304,7 @@ PetscErrorCode MatLMVMGetJ0(Mat B, Mat *J0)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMGetJ0PC - Returns a pointer to the internal PC object 
+   MatLMVMGetJ0PC - Returns a pointer to the internal PC object
    associated with the initial Jacobian.
 
    Input Parameters:
@@ -338,7 +338,7 @@ PetscErrorCode MatLMVMGetJ0PC(Mat B, PC *J0pc)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMGetJ0KSP - Returns a pointer to the internal KSP solver 
+   MatLMVMGetJ0KSP - Returns a pointer to the internal KSP solver
    associated with the initial Jacobian.
 
    Input Parameters:
@@ -368,7 +368,7 @@ PetscErrorCode MatLMVMGetJ0KSP(Mat B, KSP *J0ksp)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMApplyJ0Fwd - Applies an approximation of the forward 
+   MatLMVMApplyJ0Fwd - Applies an approximation of the forward
    matrix-vector product with the initial Jacobian.
 
    Input Parameters:
@@ -380,7 +380,7 @@ PetscErrorCode MatLMVMGetJ0KSP(Mat B, KSP *J0ksp)
 
    Level: advanced
 
-.seealso: MatLMVMSetJ0(), MatLMVMSetJ0Scale(), MatLMVMSetJ0ScaleDiag(), 
+.seealso: MatLMVMSetJ0(), MatLMVMSetJ0Scale(), MatLMVMSetJ0ScaleDiag(),
           MatLMVMSetJ0PC(), MatLMVMSetJ0KSP(), MatLMVMApplyJ0Inv()
 @*/
 PetscErrorCode MatLMVMApplyJ0Fwd(Mat B, Vec X, Vec Y)
@@ -401,7 +401,7 @@ PetscErrorCode MatLMVMApplyJ0Fwd(Mat B, Vec X, Vec Y)
   VecCheckMatCompatible(B, X, 2, Y, 3);
   if (lmvm->user_pc || lmvm->user_ksp || lmvm->J0) {
     /* User may have defined a PC or KSP for J0^{-1} so let's try to use its operators. */
-    if (lmvm->user_pc){
+    if (lmvm->user_pc) {
       ierr = PCGetOperators(lmvm->J0pc, &Amat, &Pmat);CHKERRQ(ierr);
     } else if (lmvm->user_ksp) {
       ierr = KSPGetOperators(lmvm->J0ksp, &Amat, &Pmat);CHKERRQ(ierr);
@@ -435,11 +435,11 @@ PetscErrorCode MatLMVMApplyJ0Fwd(Mat B, Vec X, Vec Y)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMApplyJ0Inv - Applies some estimation of the initial Jacobian 
-   inverse to the given vector. The specific form of the application 
-   depends on whether the user provided a scaling factor, a J0 matrix, 
-   a J0 PC, or a J0 KSP object. If no form of the initial Jacobian is 
-   provided, the function simply does an identity matrix application 
+   MatLMVMApplyJ0Inv - Applies some estimation of the initial Jacobian
+   inverse to the given vector. The specific form of the application
+   depends on whether the user provided a scaling factor, a J0 matrix,
+   a J0 PC, or a J0 KSP object. If no form of the initial Jacobian is
+   provided, the function simply does an identity matrix application
    (vector copy).
 
    Input Parameters:
@@ -451,7 +451,7 @@ PetscErrorCode MatLMVMApplyJ0Fwd(Mat B, Vec X, Vec Y)
 
    Level: advanced
 
-.seealso: MatLMVMSetJ0(), MatLMVMSetJ0Scale(), MatLMVMSetJ0ScaleDiag(), 
+.seealso: MatLMVMSetJ0(), MatLMVMSetJ0Scale(), MatLMVMSetJ0ScaleDiag(),
           MatLMVMSetJ0PC(), MatLMVMSetJ0KSP(), MatLMVMApplyJ0Fwd()
 @*/
 PetscErrorCode MatLMVMApplyJ0Inv(Mat B, Vec X, Vec Y)
@@ -500,7 +500,7 @@ PetscErrorCode MatLMVMApplyJ0Inv(Mat B, Vec X, Vec Y)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMIsAllocated - Returns a boolean flag that shows whether 
+   MatLMVMIsAllocated - Returns a boolean flag that shows whether
    the necessary data structures for the underlying matrix is allocated.
 
    Input Parameters:
@@ -519,7 +519,7 @@ PetscErrorCode MatLMVMIsAllocated(Mat B, PetscBool *flg)
   Mat_LMVM          *lmvm = (Mat_LMVM*)B->data;
   PetscErrorCode    ierr;
   PetscBool         same;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(B, MAT_CLASSID, 1);
   ierr = PetscObjectBaseTypeCompare((PetscObject)B, MATLMVM, &same);CHKERRQ(ierr);
@@ -532,10 +532,10 @@ PetscErrorCode MatLMVMIsAllocated(Mat B, PetscBool *flg)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMAllocate - Produces all necessary common memory for 
+   MatLMVMAllocate - Produces all necessary common memory for
    LMVM approximations based on the solution and function vectors
-   provided. If MatSetSizes() and MatSetUp() have not been called 
-   before MatLMVMAllocate(), the allocation will read sizes from 
+   provided. If MatSetSizes() and MatSetUp() have not been called
+   before MatLMVMAllocate(), the allocation will read sizes from
    the provided vectors and update the matrix.
 
    Input Parameters:
@@ -598,14 +598,14 @@ PetscErrorCode MatLMVMResetShift(Mat B)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMReset - Flushes all of the accumulated updates out of 
-   the LMVM approximation. In practice, this will not actually 
-   destroy the data associated with the updates. It simply resets 
-   counters, which leads to existing data being overwritten, and 
-   MatSolve() being applied as if there are no updates. A boolean 
+   MatLMVMReset - Flushes all of the accumulated updates out of
+   the LMVM approximation. In practice, this will not actually
+   destroy the data associated with the updates. It simply resets
+   counters, which leads to existing data being overwritten, and
+   MatSolve() being applied as if there are no updates. A boolean
    flag is available to force destruction of the update vectors.
-   
-   If the user has provided another LMVM matrix as J0, the J0 
+
+   If the user has provided another LMVM matrix as J0, the J0
    matrix is also reset in this function.
 
    Input Parameters:
@@ -639,7 +639,7 @@ PetscErrorCode MatLMVMReset(Mat B, PetscBool destructive)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMSetHistorySize - Set the number of past iterates to be 
+   MatLMVMSetHistorySize - Set the number of past iterates to be
    stored for the construction of the limited-memory QN update.
 
    Input Parameters:
@@ -660,7 +660,7 @@ PetscErrorCode MatLMVMSetHistorySize(Mat B, PetscInt hist_size)
   PetscErrorCode    ierr;
   PetscBool         same;
   Vec               X, F;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(B, MAT_CLASSID, 1);
   ierr = PetscObjectBaseTypeCompare((PetscObject)B, MATLMVM, &same);CHKERRQ(ierr);
@@ -675,10 +675,7 @@ PetscErrorCode MatLMVMSetHistorySize(Mat B, PetscInt hist_size)
       ierr = VecDestroy(&X);CHKERRQ(ierr);
       ierr = VecDestroy(&F);CHKERRQ(ierr);
     }
-  } else {
-    SETERRQ(PetscObjectComm((PetscObject)B), PETSC_ERR_ARG_WRONG, "QN history size must be a positive integer.");
-  }
-  
+  } else if (hist_size < 0) SETERRQ(PetscObjectComm((PetscObject)B), PETSC_ERR_ARG_WRONG, "QN history size must be a non-negative integer.");
   PetscFunctionReturn(0);
 }
 
@@ -686,8 +683,8 @@ PetscErrorCode MatLMVMSetHistorySize(Mat B, PetscInt hist_size)
 
 /*@
    MatLMVMGetUpdateCount - Returns the number of accepted updates.
-   This number may be greater than the total number of update vectors 
-   stored in the matrix. The counters are reset when MatLMVMReset() 
+   This number may be greater than the total number of update vectors
+   stored in the matrix. The counters are reset when MatLMVMReset()
    is called.
 
    Input Parameters:
@@ -705,7 +702,7 @@ PetscErrorCode MatLMVMGetUpdateCount(Mat B, PetscInt *nupdates)
   Mat_LMVM          *lmvm = (Mat_LMVM*)B->data;
   PetscErrorCode    ierr;
   PetscBool         same;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(B, MAT_CLASSID, 1);
   ierr = PetscObjectBaseTypeCompare((PetscObject)B, MATLMVM, &same);CHKERRQ(ierr);
@@ -717,7 +714,7 @@ PetscErrorCode MatLMVMGetUpdateCount(Mat B, PetscInt *nupdates)
 /*------------------------------------------------------------*/
 
 /*@
-   MatLMVMGetRejectCount - Returns the number of rejected updates. 
+   MatLMVMGetRejectCount - Returns the number of rejected updates.
    The counters are reset when MatLMVMReset() is called.
 
    Input Parameters:
@@ -735,7 +732,7 @@ PetscErrorCode MatLMVMGetRejectCount(Mat B, PetscInt *nrejects)
   Mat_LMVM          *lmvm = (Mat_LMVM*)B->data;
   PetscErrorCode    ierr;
   PetscBool         same;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(B, MAT_CLASSID, 1);
   ierr = PetscObjectBaseTypeCompare((PetscObject)B, MATLMVM, &same);CHKERRQ(ierr);
