@@ -29,7 +29,7 @@ static PetscErrorCode PCApply_Composite_Multiplicative(PC pc,Vec x,Vec y)
 
   PetscFunctionBegin;
 
-  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPC() or -pc_composite_pcs");
+  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
 
   /* Set the reuse flag on children PCs */
   while (next) {
@@ -47,7 +47,6 @@ static PetscErrorCode PCApply_Composite_Multiplicative(PC pc,Vec x,Vec y)
     next = next->next;
     ierr = MatMult(mat,y,jac->work1);CHKERRQ(ierr);                /* work1 <- A y */
     ierr = VecWAXPY(jac->work2,-1.0,jac->work1,x);CHKERRQ(ierr);   /* work2 <- x - work1 */
-    ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
     ierr = PCApply(next->pc,jac->work2,jac->work1);CHKERRQ(ierr);  /* work1 <- C work2 */
     ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);                /* y <- y + work1 = B x + C (x - A B x) = (B + C (1 - A B)) x */
   }
@@ -56,7 +55,6 @@ static PetscErrorCode PCApply_Composite_Multiplicative(PC pc,Vec x,Vec y)
       next = next->previous;
       ierr = MatMult(mat,y,jac->work1);CHKERRQ(ierr);
       ierr = VecWAXPY(jac->work2,-1.0,jac->work1,x);CHKERRQ(ierr);
-      ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
       ierr = PCApply(next->pc,jac->work2,jac->work1);CHKERRQ(ierr);
       ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);
     }
@@ -72,7 +70,7 @@ static PetscErrorCode PCApplyTranspose_Composite_Multiplicative(PC pc,Vec x,Vec
   Mat              mat  = pc->pmat;
 
   PetscFunctionBegin;
-  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPC() or -pc_composite_pcs");
+  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
   if (next->next && !jac->work2) { /* allocate second work vector */
     ierr = VecDuplicate(jac->work1,&jac->work2);CHKERRQ(ierr);
   }
@@ -86,7 +84,6 @@ static PetscErrorCode PCApplyTranspose_Composite_Multiplicative(PC pc,Vec x,Vec
     next = next->previous;
     ierr = MatMultTranspose(mat,y,jac->work1);CHKERRQ(ierr);
     ierr = VecWAXPY(jac->work2,-1.0,jac->work1,x);CHKERRQ(ierr);
-    ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
     ierr = PCApplyTranspose(next->pc,jac->work2,jac->work1);CHKERRQ(ierr);
     ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);
   }
@@ -96,7 +93,6 @@ static PetscErrorCode PCApplyTranspose_Composite_Multiplicative(PC pc,Vec x,Vec
       next = next->next;
       ierr = MatMultTranspose(mat,y,jac->work1);CHKERRQ(ierr);
       ierr = VecWAXPY(jac->work2,-1.0,jac->work1,x);CHKERRQ(ierr);
-      ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
       ierr = PCApplyTranspose(next->pc,jac->work2,jac->work1);CHKERRQ(ierr);
       ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);
     }
@@ -116,7 +112,7 @@ static PetscErrorCode PCApply_Composite_Special(PC pc,Vec x,Vec y)
   PC_CompositeLink next = jac->head;
 
   PetscFunctionBegin;
-  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPC() or -pc_composite_pcs");
+  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
   if (!next->next || next->next->next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Special composite preconditioners requires exactly two PCs");
 
   /* Set the reuse flag on children PCs */
@@ -135,7 +131,7 @@ static PetscErrorCode PCApply_Composite_Additive(PC pc,Vec x,Vec y)
   PC_CompositeLink next = jac->head;
 
   PetscFunctionBegin;
-  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPC() or -pc_composite_pcs");
+  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
 
   /* Set the reuse flag on children PCs */
   while (next) {
@@ -147,7 +143,6 @@ static PetscErrorCode PCApply_Composite_Additive(PC pc,Vec x,Vec y)
   ierr = PCApply(next->pc,x,y);CHKERRQ(ierr);
   while (next->next) {
     next = next->next;
-    ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
     ierr = PCApply(next->pc,x,jac->work1);CHKERRQ(ierr);
     ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);
   }
@@ -161,11 +156,10 @@ static PetscErrorCode PCApplyTranspose_Composite_Additive(PC pc,Vec x,Vec y)
   PC_CompositeLink next = jac->head;
 
   PetscFunctionBegin;
-  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPC() or -pc_composite_pcs");
+  if (!next) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
   ierr = PCApplyTranspose(next->pc,x,y);CHKERRQ(ierr);
   while (next->next) {
     next = next->next;
-    ierr = VecSet(jac->work1,0.0);CHKERRQ(ierr);  /* zero since some PC's may not set all entries in the result */
     ierr = PCApplyTranspose(next->pc,x,jac->work1);CHKERRQ(ierr);
     ierr = VecAXPY(y,1.0,jac->work1);CHKERRQ(ierr);
   }
@@ -181,12 +175,16 @@ static PetscErrorCode PCSetUp_Composite(PC pc)
 
   PetscFunctionBegin;
   if (!jac->work1) {
-    ierr = MatCreateVecs(pc->pmat,&jac->work1,0);CHKERRQ(ierr);
+    ierr = MatCreateVecs(pc->pmat,&jac->work1,NULL);CHKERRQ(ierr);
   }
   ierr = PCGetDM(pc,&dm);CHKERRQ(ierr);
   while (next) {
-    ierr = PCSetDM(next->pc,dm);CHKERRQ(ierr);
-    ierr = PCSetOperators(next->pc,pc->mat,pc->pmat);CHKERRQ(ierr);
+    if (!next->pc->dm) {
+      ierr = PCSetDM(next->pc,dm);CHKERRQ(ierr);
+    }
+    if (!next->pc->mat) {
+      ierr = PCSetOperators(next->pc,pc->mat,pc->pmat);CHKERRQ(ierr);
+    }
     next = next->next;
   }
   PetscFunctionReturn(0);
@@ -241,10 +239,10 @@ static PetscErrorCode PCSetFromOptions_Composite(PetscOptionItems *PetscOptionsO
   if (flg) {
     ierr = PCCompositeSetType(pc,jac->type);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsStringArray("-pc_composite_pcs","List of composite solvers","PCCompositeAddPC",pcs,&nmax,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsStringArray("-pc_composite_pcs","List of composite solvers","PCCompositeAddPCType",pcs,&nmax,&flg);CHKERRQ(ierr);
   if (flg) {
     for (i=0; i<nmax; i++) {
-      ierr = PCCompositeAddPC(pc,pcs[i]);CHKERRQ(ierr);
+      ierr = PCCompositeAddPCType(pc,pcs[i]);CHKERRQ(ierr);
       ierr = PetscFree(pcs[i]);CHKERRQ(ierr);   /* deallocate string pcs[i], which is allocated in PetscOptionsStringArray() */
     }
   }
@@ -311,7 +309,7 @@ static PetscErrorCode  PCCompositeSetType_Composite(PC pc,PCCompositeType type)
   } else if (type ==  PC_COMPOSITE_SPECIAL) {
     pc->ops->apply          = PCApply_Composite_Special;
     pc->ops->applytranspose = NULL;
-  } else SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Unkown composite preconditioner type");
+  } else SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Unknown composite preconditioner type");
   jac->type = type;
   PetscFunctionReturn(0);
 }
@@ -325,22 +323,21 @@ static PetscErrorCode  PCCompositeGetType_Composite(PC pc,PCCompositeType *type)
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode  PCCompositeAddPC_Composite(PC pc,PCType type)
+static PetscErrorCode PCCompositeAddPC_Composite(PC pc, PC subpc)
 {
-  PC_Composite     *jac;
-  PC_CompositeLink next,ilink;
-  PetscErrorCode   ierr;
+  PC_Composite    *jac;
+  PC_CompositeLink next, ilink;
   PetscInt         cnt = 0;
-  const char       *prefix;
+  const char      *prefix;
   char             newprefix[20];
+  PetscErrorCode   ierr;
 
   PetscFunctionBegin;
-  ierr        = PetscNewLog(pc,&ilink);CHKERRQ(ierr);
-  ilink->next = 0;
-  ierr        = PCCreate(PetscObjectComm((PetscObject)pc),&ilink->pc);CHKERRQ(ierr);
-  ierr        = PetscLogObjectParent((PetscObject)pc,(PetscObject)ilink->pc);CHKERRQ(ierr);
+  ierr = PetscNewLog(pc, &ilink);CHKERRQ(ierr);
+  ilink->next = NULL;
+  ilink->pc   = subpc;
 
-  jac  = (PC_Composite*)pc->data;
+  jac  = (PC_Composite *) pc->data;
   next = jac->head;
   if (!next) {
     jac->head       = ilink;
@@ -354,12 +351,27 @@ static PetscErrorCode  PCCompositeAddPC_Composite(PC pc,PCType type)
     next->next      = ilink;
     ilink->previous = next;
   }
-  ierr = PCGetOptionsPrefix(pc,&prefix);CHKERRQ(ierr);
-  ierr = PCSetOptionsPrefix(ilink->pc,prefix);CHKERRQ(ierr);
-  sprintf(newprefix,"sub_%d_",(int)cnt);
-  ierr = PCAppendOptionsPrefix(ilink->pc,newprefix);CHKERRQ(ierr);
+  ierr = PCGetOptionsPrefix(pc, &prefix);CHKERRQ(ierr);
+  ierr = PCSetOptionsPrefix(subpc, prefix);CHKERRQ(ierr);
+  ierr = PetscSNPrintf(newprefix, 20, "sub_%d_", (int) cnt);CHKERRQ(ierr);
+  ierr = PCAppendOptionsPrefix(subpc, newprefix);CHKERRQ(ierr);
+  ierr = PetscObjectReference((PetscObject) subpc);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCCompositeAddPCType_Composite(PC pc, PCType type)
+{
+  PC             subpc;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PCCreate(PetscObjectComm((PetscObject)pc), &subpc);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject) subpc, (PetscObject) pc, 1);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject) pc, (PetscObject) subpc);CHKERRQ(ierr);
+  ierr = PCCompositeAddPC_Composite(pc, subpc);CHKERRQ(ierr);
   /* type is set after prefix, because some methods may modify prefix, e.g. pcksp */
-  ierr = PCSetType(ilink->pc,type);CHKERRQ(ierr);
+  ierr = PCSetType(subpc, type);CHKERRQ(ierr);
+  ierr = PCDestroy(&subpc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -411,7 +423,6 @@ static PetscErrorCode  PCCompositeGetPC_Composite(PC pc,PetscInt n,PC *subpc)
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeSetType(PC pc,PCCompositeType type)
 {
@@ -440,7 +451,6 @@ PetscErrorCode  PCCompositeSetType(PC pc,PCCompositeType type)
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeGetType(PC pc,PCCompositeType *type)
 {
@@ -458,13 +468,12 @@ PetscErrorCode  PCCompositeGetType(PC pc,PCCompositeType *type)
 
    Logically Collective on PC
 
-   Input Parameter:
+   Input Parameters:
 +  pc - the preconditioner context
 -  alpha - scale on identity
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeSpecialSetAlpha(PC pc,PetscScalar alpha)
 {
@@ -478,25 +487,49 @@ PetscErrorCode  PCCompositeSpecialSetAlpha(PC pc,PetscScalar alpha)
 }
 
 /*@C
-   PCCompositeAddPC - Adds another PC to the composite PC.
+  PCCompositeAddPCType - Adds another PC of the given type to the composite PC.
 
-   Collective on PC
+  Collective on PC
 
-   Input Parameters:
-+  pc - the preconditioner context
--  type - the type of the new preconditioner
+  Input Parameters:
++ pc - the preconditioner context
+- type - the type of the new preconditioner
 
-   Level: Developer
+  Level: Developer
 
-.keywords: PC, composite preconditioner, add
+.seealso: PCCompositeAddPC()
 @*/
-PetscErrorCode  PCCompositeAddPC(PC pc,PCType type)
+PetscErrorCode  PCCompositeAddPCType(PC pc,PCType type)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscTryMethod(pc,"PCCompositeAddPC_C",(PC,PCType),(pc,type));CHKERRQ(ierr);
+  ierr = PetscTryMethod(pc,"PCCompositeAddPCType_C",(PC,PCType),(pc,type));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+  PCCompositeAddPC - Adds another PC to the composite PC.
+
+  Collective on PC
+
+  Input Parameters:
++ pc    - the preconditioner context
+- subpc - the new preconditioner
+
+   Level: Developer
+
+.seealso: PCCompositeAddPCType()
+@*/
+PetscErrorCode PCCompositeAddPC(PC pc, PC subpc)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidHeaderSpecific(subpc,PC_CLASSID,2);
+  ierr = PetscTryMethod(pc,"PCCompositeAddPC_C",(PC,PC),(pc,subpc));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -513,8 +546,6 @@ PetscErrorCode  PCCompositeAddPC(PC pc,PCType type)
 
    Level: Developer
 
-.keywords: PC, get, composite preconditioner, sub preconditioner
-
 .seealso: PCCompositeGetPC()
 @*/
 PetscErrorCode  PCCompositeGetNumberPC(PC pc,PetscInt *num)
@@ -533,7 +564,7 @@ PetscErrorCode  PCCompositeGetNumberPC(PC pc,PetscInt *num)
 
    Not Collective
 
-   Input Parameter:
+   Input Parameters:
 +  pc - the preconditioner context
 -  n - the number of the pc requested
 
@@ -542,11 +573,13 @@ PetscErrorCode  PCCompositeGetNumberPC(PC pc,PetscInt *num)
 
    Level: Developer
 
-.keywords: PC, get, composite preconditioner, sub preconditioner
+    Notes:
+    To use a different operator to construct one of the inner preconditioners first call PCCompositeGetPC(), then
+            call PCSetOperators() on that PC.
 
-.seealso: PCCompositeAddPC(), PCCompositeGetNumberPC()
+.seealso: PCCompositeAddPCType(), PCCompositeGetNumberPC(), PCSetOperators()
 @*/
-PetscErrorCode  PCCompositeGetPC(PC pc,PetscInt n,PC *subpc)
+PetscErrorCode PCCompositeGetPC(PC pc,PetscInt n,PC *subpc)
 {
   PetscErrorCode ierr;
 
@@ -564,21 +597,21 @@ PetscErrorCode  PCCompositeGetPC(PC pc,PetscInt n,PC *subpc)
 
    Options Database Keys:
 +  -pc_composite_type <type: one of multiplicative, additive, symmetric_multiplicative, special> - Sets composite preconditioner type
-.  -pc_use_amat - Activates PCSetUseAmat()
+.  -pc_use_amat - activates PCSetUseAmat()
 -  -pc_composite_pcs - <pc0,pc1,...> list of PCs to compose
 
    Level: intermediate
 
-   Concepts: composing solvers
-
-   Notes: To use a Krylov method inside the composite preconditioner, set the PCType of one or more
+   Notes:
+    To use a Krylov method inside the composite preconditioner, set the PCType of one or more
           inner PCs to be PCKSP.
           Using a Krylov method inside another Krylov method can be dangerous (you get divergence or
           the incorrect answer) unless you use KSPFGMRES as the outer Krylov method
-
+          To use a different operator to construct one of the inner preconditioners first call PCCompositeGetPC(), then
+          call PCSetOperators() on that PC.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
-           PCSHELL, PCKSP, PCCompositeSetType(), PCCompositeSpecialSetAlpha(), PCCompositeAddPC(),
+           PCSHELL, PCKSP, PCCompositeSetType(), PCCompositeSpecialSetAlpha(), PCCompositeAddPCType(),
            PCCompositeGetPC(), PCSetUseAmat()
 
 M*/
@@ -598,20 +631,20 @@ PETSC_EXTERN PetscErrorCode PCCreate_Composite(PC pc)
   pc->ops->destroy         = PCDestroy_Composite;
   pc->ops->setfromoptions  = PCSetFromOptions_Composite;
   pc->ops->view            = PCView_Composite;
-  pc->ops->applyrichardson = 0;
+  pc->ops->applyrichardson = NULL;
 
   pc->data   = (void*)jac;
   jac->type  = PC_COMPOSITE_ADDITIVE;
-  jac->work1 = 0;
-  jac->work2 = 0;
-  jac->head  = 0;
+  jac->work1 = NULL;
+  jac->work2 = NULL;
+  jac->head  = NULL;
 
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeSetType_C",PCCompositeSetType_Composite);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeGetType_C",PCCompositeGetType_Composite);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeAddPCType_C",PCCompositeAddPCType_Composite);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeAddPC_C",PCCompositeAddPC_Composite);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeGetNumberPC_C",PCCompositeGetNumberPC_Composite);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeGetPC_C",PCCompositeGetPC_Composite);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCCompositeSpecialSetAlpha_C",PCCompositeSpecialSetAlpha_Composite);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
