@@ -335,13 +335,12 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
   char           **subdomain_names = NULL;
   PetscInt       *numbering;
 
-
   PetscFunctionBegin;
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&size);CHKERRMPI(ierr);
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)pc),&rank);CHKERRMPI(ierr);
   if (!pc->setupcalled) {
         /* use a hierarchical partitioning */
-    if (osm->hierarchicalpartitioning){
+    if (osm->hierarchicalpartitioning) {
       ierr = PCGASMSetHierarchicalPartitioning(pc);CHKERRQ(ierr);
     }
     if (osm->n == PETSC_DETERMINE) {
@@ -404,7 +403,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       PetscMPIInt inwork,outwork;
       /* determine global number of subdomains and the max number of local subdomains */
       inwork     = osm->n;
-      ierr       = MPIU_Allreduce(&inwork,&outwork,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)pc));CHKERRQ(ierr);
+      ierr       = MPIU_Allreduce(&inwork,&outwork,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)pc));CHKERRMPI(ierr);
       osm->nmax  = outwork;
     }
     if (osm->N == PETSC_DETERMINE) {
@@ -442,7 +441,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
     }
     ierr = ISCreateGeneral(((PetscObject)(pc))->comm,on,oidx,PETSC_OWN_POINTER,&gois);CHKERRQ(ierr);
     nTotalInnerIndices = 0;
-    for (i=0; i<osm->n; i++){
+    for (i=0; i<osm->n; i++) {
       ierr = ISGetLocalSize(osm->iis[i],&nInnerIndices);CHKERRQ(ierr);
       nTotalInnerIndices += nInnerIndices;
     }
@@ -483,7 +482,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       in   = 0;
       for (i=0; i<osm->n; i++) {
         ierr   = ISGetLocalSize(osm->iis[i],&ini);CHKERRQ(ierr);
-        for (k = 0; k < ini; ++k){
+        for (k = 0; k < ini; ++k) {
           array[in+k] = numbering[i];
         }
         in += ini;
@@ -568,7 +567,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       ierr  = MatDestroyMatrices(osm->n,&osm->pmat);CHKERRQ(ierr);
       scall = MAT_INITIAL_MATRIX;
     }
-    if (osm->permutationIS){
+    if (osm->permutationIS) {
       ierr = MatCreateSubMatrix(pc->pmat,osm->permutationIS,osm->permutationIS,scall,&osm->permutationP);CHKERRQ(ierr);
       ierr = PetscObjectReference((PetscObject)osm->permutationP);CHKERRQ(ierr);
       osm->pcmat = pc->pmat;
@@ -607,7 +606,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       ierr = KSPSetFromOptions(osm->ksp[i]);CHKERRQ(ierr);
     }
   }
-  if (osm->pcmat){
+  if (osm->pcmat) {
     ierr = MatDestroy(&pc->pmat);CHKERRQ(ierr);
     pc->pmat   = osm->pcmat;
     osm->pcmat = NULL;
@@ -701,12 +700,12 @@ static PetscErrorCode PCMatApply_GASM(PC pc,Mat Xin,Mat Yout)
     ierr = VecGetSize(osm->pcx,&M);CHKERRQ(ierr);
     ierr = MatCreateDense(PetscObjectComm((PetscObject)osm->ois[0]),m,PETSC_DECIDE,M,N,NULL,&O);CHKERRQ(ierr);
     for (i = 0; i < N; ++i) {
-      ierr = MatDenseGetColumnVecRead(Xin,i,&x);
-      ierr = MatDenseGetColumnVecWrite(O,i,&y);
+      ierr = MatDenseGetColumnVecRead(Xin,i,&x);CHKERRQ(ierr);
+      ierr = MatDenseGetColumnVecWrite(O,i,&y);CHKERRQ(ierr);
       ierr = VecScatterBegin(osm->pctoouter,x,y,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->pctoouter,x,y,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-      ierr = MatDenseRestoreColumnVecWrite(O,i,&y);
-      ierr = MatDenseRestoreColumnVecRead(Xin,i,&x);
+      ierr = MatDenseRestoreColumnVecWrite(O,i,&y);CHKERRQ(ierr);
+      ierr = MatDenseRestoreColumnVecRead(Xin,i,&x);CHKERRQ(ierr);
     }
     X = Y = O;
   } else {
@@ -721,8 +720,8 @@ static PetscErrorCode PCMatApply_GASM(PC pc,Mat Xin,Mat Yout)
   ierr = VecGetSize(osm->x[0],&M);CHKERRQ(ierr);
   ierr = MatCreateDense(PetscObjectComm((PetscObject)osm->ois[0]),m,PETSC_DECIDE,M,N,NULL,&Z);CHKERRQ(ierr);
   for (i = 0; i < N; ++i) {
-    ierr = MatDenseGetColumnVecRead(X,i,&x);
-    ierr = MatDenseGetColumnVecWrite(Z,i,&y);
+    ierr = MatDenseGetColumnVecRead(X,i,&x);CHKERRQ(ierr);
+    ierr = MatDenseGetColumnVecWrite(Z,i,&y);CHKERRQ(ierr);
     if (!(osm->type & PC_GASM_RESTRICT)) {
       /* have to zero the work RHS since scatter may leave some slots empty */
       ierr = VecZeroEntries(y);CHKERRQ(ierr);
@@ -732,8 +731,8 @@ static PetscErrorCode PCMatApply_GASM(PC pc,Mat Xin,Mat Yout)
       ierr = VecScatterBegin(osm->gorestriction,x,y,INSERT_VALUES,forward);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->gorestriction,x,y,INSERT_VALUES,forward);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreColumnVecWrite(Z,i,&y);
-    ierr = MatDenseRestoreColumnVecRead(X,i,&x);
+    ierr = MatDenseRestoreColumnVecWrite(Z,i,&y);CHKERRQ(ierr);
+    ierr = MatDenseRestoreColumnVecRead(X,i,&x);CHKERRQ(ierr);
   }
   ierr = MatCreateDense(PetscObjectComm((PetscObject)osm->ois[0]),m,PETSC_DECIDE,M,N,NULL,&W);CHKERRQ(ierr);
   ierr = MatSetOption(Z,MAT_NO_OFF_PROC_ENTRIES,PETSC_TRUE);CHKERRQ(ierr);
@@ -746,8 +745,8 @@ static PetscErrorCode PCMatApply_GASM(PC pc,Mat Xin,Mat Yout)
   /* do we need to zero y? */
   ierr = MatZeroEntries(Y);CHKERRQ(ierr);
   for (i = 0; i < N; ++i) {
-    ierr = MatDenseGetColumnVecWrite(Y,i,&y);
-    ierr = MatDenseGetColumnVecRead(W,i,&x);
+    ierr = MatDenseGetColumnVecWrite(Y,i,&y);CHKERRQ(ierr);
+    ierr = MatDenseGetColumnVecRead(W,i,&x);CHKERRQ(ierr);
     if (!(osm->type & PC_GASM_INTERPOLATE)) {
       ierr = VecScatterBegin(osm->girestriction,x,y,ADD_VALUES,reverse);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->girestriction,x,y,ADD_VALUES,reverse);CHKERRQ(ierr);
@@ -755,14 +754,14 @@ static PetscErrorCode PCMatApply_GASM(PC pc,Mat Xin,Mat Yout)
       ierr = VecScatterBegin(osm->gorestriction,x,y,ADD_VALUES,reverse);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->gorestriction,x,y,ADD_VALUES,reverse);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreColumnVecRead(W,i,&x);
+    ierr = MatDenseRestoreColumnVecRead(W,i,&x);CHKERRQ(ierr);
     if (osm->pctoouter) {
-      ierr = MatDenseGetColumnVecWrite(Yout,i,&x);
+      ierr = MatDenseGetColumnVecWrite(Yout,i,&x);CHKERRQ(ierr);
       ierr = VecScatterBegin(osm->pctoouter,y,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->pctoouter,y,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-      ierr = MatDenseRestoreColumnVecRead(Yout,i,&x);
+      ierr = MatDenseRestoreColumnVecRead(Yout,i,&x);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreColumnVecWrite(Y,i,&y);
+    ierr = MatDenseRestoreColumnVecWrite(Y,i,&y);CHKERRQ(ierr);
   }
   ierr = MatDestroy(&W);CHKERRQ(ierr);
   ierr = MatDestroy(&O);CHKERRQ(ierr);
@@ -778,7 +777,7 @@ static PetscErrorCode PCApplyTranspose_GASM(PC pc,Vec xin,Vec yout)
   ScatterMode    forward = SCATTER_FORWARD,reverse = SCATTER_REVERSE;
 
   PetscFunctionBegin;
-  if (osm->pctoouter){
+  if (osm->pctoouter) {
    ierr = VecScatterBegin(osm->pctoouter,xin,osm->pcx,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
    ierr = VecScatterEnd(osm->pctoouter,xin,osm->pcx,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
    x = osm->pcx;
@@ -820,7 +819,7 @@ static PetscErrorCode PCApplyTranspose_GASM(PC pc,Vec xin,Vec yout)
     ierr = VecScatterBegin(osm->gorestriction,osm->gy,y,ADD_VALUES,reverse);CHKERRQ(ierr);
     ierr = VecScatterEnd(osm->gorestriction,osm->gy,y,ADD_VALUES,reverse);CHKERRQ(ierr);
   }
-  if (osm->pctoouter){
+  if (osm->pctoouter) {
    ierr = VecScatterBegin(osm->pctoouter,y,yout,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
    ierr = VecScatterEnd(osm->pctoouter,y,yout,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   }
@@ -867,22 +866,22 @@ static PetscErrorCode PCReset_GASM(PC pc)
     osm->N    = PETSC_DETERMINE;
     osm->nmax = PETSC_DETERMINE;
   }
-  if (osm->pctoouter){
+  if (osm->pctoouter) {
         ierr = VecScatterDestroy(&(osm->pctoouter));CHKERRQ(ierr);
   }
-  if (osm->permutationIS){
+  if (osm->permutationIS) {
         ierr = ISDestroy(&(osm->permutationIS));CHKERRQ(ierr);
   }
-  if (osm->pcx){
+  if (osm->pcx) {
         ierr = VecDestroy(&(osm->pcx));CHKERRQ(ierr);
   }
-  if (osm->pcy){
+  if (osm->pcy) {
         ierr = VecDestroy(&(osm->pcy));CHKERRQ(ierr);
   }
-  if (osm->permutationP){
+  if (osm->permutationP) {
     ierr = MatDestroy(&(osm->permutationP));CHKERRQ(ierr);
   }
-  if (osm->pcmat){
+  if (osm->pcmat) {
         ierr = MatDestroy(&osm->pcmat);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -949,7 +948,6 @@ static PetscErrorCode PCSetFromOptions_GASM(PetscOptionItems *PetscOptionsObject
 +   pc  - the preconditioner
 -   N   - total number of subdomains
 
-
     Level: beginner
 
 .seealso: PCGASMSetSubdomains(), PCGASMSetOverlap()
@@ -1127,10 +1125,9 @@ static PetscErrorCode  PCGASMGetSubKSP_GASM(PC pc,PetscInt *n,PetscInt *first,KS
 
     By default the GASM preconditioner uses 1 (local) subdomain per processor.
 
-
     Level: advanced
 
-.seealso: PCGASMSetNumSubdomains(), PCGASMSetOverlap(), PCGASMGetSubKSP(),
+.seealso: PCGASMSetOverlap(), PCGASMGetSubKSP(),
           PCGASMCreateSubdomains2D(), PCGASMGetSubdomains()
 @*/
 PetscErrorCode  PCGASMSetSubdomains(PC pc,PetscInt n,IS iis[],IS ois[])
@@ -1317,7 +1314,6 @@ PetscErrorCode  PCGASMGetSubKSP(PC pc,PetscInt *n_local,PetscInt *first_local,KS
          and set the options directly on the resulting KSP object (you can access its PC
          with KSPGetPC())
 
-
    Level: beginner
 
     References:
@@ -1564,7 +1560,6 @@ PETSC_INTERN PetscErrorCode  PCGASMCreateStraddlingSubdomains(Mat A,PetscInt N,P
          outer subdomains will be automatically generated from these according to the requested amount of
          overlap; this is currently supported only with local subdomains.
 
-
 .seealso: PCGASMSetSubdomains(), PCGASMDestroySubdomains()
 @*/
 PetscErrorCode  PCGASMCreateSubdomains(Mat A,PetscInt N,PetscInt *n,IS *iis[])
@@ -1673,17 +1668,19 @@ PetscErrorCode  PCGASMDestroySubdomains(PetscInt n,IS **iis,IS **ois)
    Collective
 
    Input Parameters:
-+  M, N               - the global number of grid points in the x and y directions
-.  Mdomains, Ndomains - the global number of subdomains in the x and y directions
-.  dof                - degrees of freedom per node
--  overlap            - overlap in mesh lines
++  pc       - the preconditioner context
+.  M        - the global number of grid points in the x direction
+.  N        - the global number of grid points in the y direction
+.  Mdomains - the global number of subdomains in the x direction
+.  Ndomains - the global number of subdomains in the y direction
+.  dof      - degrees of freedom per node
+-  overlap  - overlap in mesh lines
 
    Output Parameters:
 +  Nsub - the number of local subdomains created
 .  iis  - array of index sets defining inner (nonoverlapping) subdomains
 -  ois  - array of index sets defining outer (overlapping, if overlap > 0) subdomains
 
-
    Level: advanced
 
 .seealso: PCGASMSetSubdomains(), PCGASMGetSubKSP(), PCGASMSetOverlap()
@@ -1855,7 +1852,6 @@ PetscErrorCode  PCGASMCreateSubdomains2D(PC pc,PetscInt M,PetscInt N,PetscInt Md
 .   iis - the index sets that define the inner subdomains (without overlap) supported on this processor (can be NULL)
 -   ois - the index sets that define the outer subdomains (with overlap) supported on this processor (can be NULL)
 
-
     Notes:
     The user is responsible for destroying the ISs and freeing the returned arrays.
     The IS numbering is in the parallel, global numbering of the vector.
@@ -1937,7 +1933,7 @@ PetscErrorCode  PCGASMGetSubmatrices(PC pc,PetscInt *n,Mat *mat[])
     PCGASMSetUseDMSubdomains - Indicates whether to use DMCreateDomainDecomposition() to define the subdomains, whenever possible.
     Logically Collective
 
-    Input Parameter:
+    Input Parameters:
 +   pc  - the preconditioner
 -   flg - boolean indicating whether to use subdomains defined by the DM
 
