@@ -50,7 +50,7 @@
 
 #include <petsc/private/pcimpl.h>   /*I "petscpc.h" I*/
 
-const char *const PCJacobiTypes[]    = {"DIAGONAL","ROWMAX","ROWSUM","PCJacobiType","PC_JACOBI_",0};
+const char *const PCJacobiTypes[]    = {"DIAGONAL","ROWMAX","ROWSUM","PCJacobiType","PC_JACOBI_",NULL};
 
 /*
    Private context (data structure) for the Jacobi preconditioner.
@@ -63,10 +63,9 @@ typedef struct {
   PetscBool userowmax;           /* set with PCJacobiSetType() */
   PetscBool userowsum;
   PetscBool useabs;              /* use the absolute values of the diagonal entries */
+  PetscBool fixdiag;             /* fix zero diagonal terms */
 } PC_Jacobi;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiSetType_Jacobi"
 static PetscErrorCode  PCJacobiSetType_Jacobi(PC pc,PCJacobiType type)
 {
   PC_Jacobi *j = (PC_Jacobi*)pc->data;
@@ -82,8 +81,6 @@ static PetscErrorCode  PCJacobiSetType_Jacobi(PC pc,PCJacobiType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiGetType_Jacobi"
 static PetscErrorCode  PCJacobiGetType_Jacobi(PC pc,PCJacobiType *type)
 {
   PC_Jacobi *j = (PC_Jacobi*)pc->data;
@@ -99,8 +96,6 @@ static PetscErrorCode  PCJacobiGetType_Jacobi(PC pc,PCJacobiType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiSetUseAbs_Jacobi"
 static PetscErrorCode  PCJacobiSetUseAbs_Jacobi(PC pc,PetscBool flg)
 {
   PC_Jacobi *j = (PC_Jacobi*)pc->data;
@@ -110,8 +105,6 @@ static PetscErrorCode  PCJacobiSetUseAbs_Jacobi(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiGetUseAbs_Jacobi"
 static PetscErrorCode  PCJacobiGetUseAbs_Jacobi(PC pc,PetscBool *flg)
 {
   PC_Jacobi *j = (PC_Jacobi*)pc->data;
@@ -121,6 +114,24 @@ static PetscErrorCode  PCJacobiGetUseAbs_Jacobi(PC pc,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode  PCJacobiSetFixDiagonal_Jacobi(PC pc,PetscBool flg)
+{
+  PC_Jacobi *j = (PC_Jacobi*)pc->data;
+
+  PetscFunctionBegin;
+  j->fixdiag = flg;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode  PCJacobiGetFixDiagonal_Jacobi(PC pc,PetscBool *flg)
+{
+  PC_Jacobi *j = (PC_Jacobi*)pc->data;
+
+  PetscFunctionBegin;
+  *flg = j->fixdiag;
+  PetscFunctionReturn(0);
+}
+
 /* -------------------------------------------------------------------------- */
 /*
    PCSetUp_Jacobi - Prepares for the use of the Jacobi preconditioner
@@ -135,8 +146,6 @@ static PetscErrorCode  PCJacobiGetUseAbs_Jacobi(PC pc,PetscBool *flg)
    The interface routine PCSetUp() is not usually called directly by
    the user, but instead is called by PCApply() if necessary.
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Jacobi"
 static PetscErrorCode PCSetUp_Jacobi(PC pc)
 {
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
@@ -172,6 +181,8 @@ static PetscErrorCode PCSetUp_Jacobi(PC pc)
   diagsqrt = jac->diagsqrt;
 
   if (diag) {
+    PetscBool isspd;
+
     if (jac->userowmax) {
       ierr = MatGetRowMaxAbs(pc->pmat,diag,NULL);CHKERRQ(ierr);
     } else if (jac->userowsum) {
@@ -180,18 +191,21 @@ static PetscErrorCode PCSetUp_Jacobi(PC pc)
       ierr = MatGetDiagonal(pc->pmat,diag);CHKERRQ(ierr);
     }
     ierr = VecReciprocal(diag);CHKERRQ(ierr);
-    ierr = VecGetLocalSize(diag,&n);CHKERRQ(ierr);
-    ierr = VecGetArray(diag,&x);CHKERRQ(ierr);
     if (jac->useabs) {
-      for (i=0; i<n; i++) x[i] = PetscAbsScalar(x[i]);
+      ierr = VecAbs(diag);CHKERRQ(ierr);
     }
-    for (i=0; i<n; i++) {
-      if (x[i] == 0.0) {
-        x[i]     = 1.0;
-        zeroflag = PETSC_TRUE;
+    ierr = MatGetOption(pc->pmat,MAT_SPD,&isspd);CHKERRQ(ierr);
+    if (jac->fixdiag && !isspd) {
+      ierr = VecGetLocalSize(diag,&n);CHKERRQ(ierr);
+      ierr = VecGetArray(diag,&x);CHKERRQ(ierr);
+      for (i=0; i<n; i++) {
+        if (x[i] == 0.0) {
+          x[i]     = 1.0;
+          zeroflag = PETSC_TRUE;
+        }
       }
+      ierr = VecRestoreArray(diag,&x);CHKERRQ(ierr);
     }
-    ierr = VecRestoreArray(diag,&x);CHKERRQ(ierr);
   }
   if (diagsqrt) {
     if (jac->userowmax) {
@@ -226,15 +240,13 @@ static PetscErrorCode PCSetUp_Jacobi(PC pc)
    Input Parameter:
 .  pc - the preconditioner context
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Jacobi_Symmetric"
 static PetscErrorCode PCSetUp_Jacobi_Symmetric(PC pc)
 {
   PetscErrorCode ierr;
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
 
   PetscFunctionBegin;
-  ierr = MatCreateVecs(pc->pmat,&jac->diagsqrt,0);CHKERRQ(ierr);
+  ierr = MatCreateVecs(pc->pmat,&jac->diagsqrt,NULL);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)pc,(PetscObject)jac->diagsqrt);CHKERRQ(ierr);
   ierr = PCSetUp_Jacobi(pc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -248,15 +260,13 @@ static PetscErrorCode PCSetUp_Jacobi_Symmetric(PC pc)
    Input Parameter:
 .  pc - the preconditioner context
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Jacobi_NonSymmetric"
 static PetscErrorCode PCSetUp_Jacobi_NonSymmetric(PC pc)
 {
   PetscErrorCode ierr;
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
 
   PetscFunctionBegin;
-  ierr = MatCreateVecs(pc->pmat,&jac->diag,0);CHKERRQ(ierr);
+  ierr = MatCreateVecs(pc->pmat,&jac->diag,NULL);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)pc,(PetscObject)jac->diag);CHKERRQ(ierr);
   ierr = PCSetUp_Jacobi(pc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -274,8 +284,6 @@ static PetscErrorCode PCSetUp_Jacobi_NonSymmetric(PC pc)
 
    Application Interface Routine: PCApply()
  */
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_Jacobi"
 static PetscErrorCode PCApply_Jacobi(PC pc,Vec x,Vec y)
 {
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
@@ -302,8 +310,6 @@ static PetscErrorCode PCApply_Jacobi(PC pc,Vec x,Vec y)
 
    Application Interface Routines: PCApplySymmetricLeft(), PCApplySymmetricRight()
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricLeftOrRight_Jacobi"
 static PetscErrorCode PCApplySymmetricLeftOrRight_Jacobi(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -313,12 +319,11 @@ static PetscErrorCode PCApplySymmetricLeftOrRight_Jacobi(PC pc,Vec x,Vec y)
   if (!jac->diagsqrt) {
     ierr = PCSetUp_Jacobi_Symmetric(pc);CHKERRQ(ierr);
   }
-  VecPointwiseMult(y,x,jac->diagsqrt);
+  ierr = VecPointwiseMult(y,x,jac->diagsqrt);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
 /* -------------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_Jacobi"
 static PetscErrorCode PCReset_Jacobi(PC pc)
 {
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
@@ -339,14 +344,18 @@ static PetscErrorCode PCReset_Jacobi(PC pc)
 
    Application Interface Routine: PCDestroy()
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_Jacobi"
 static PetscErrorCode PCDestroy_Jacobi(PC pc)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PCReset_Jacobi(pc);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiSetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiSetUseAbs_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetUseAbs_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiSetFixDiagonal_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetFixDiagonal_C",NULL);CHKERRQ(ierr);
 
   /*
       Free the private data structure that was hanging off the PC
@@ -355,8 +364,6 @@ static PetscErrorCode PCDestroy_Jacobi(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_Jacobi"
 static PetscErrorCode PCSetFromOptions_Jacobi(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_Jacobi      *jac = (PC_Jacobi*)pc->data;
@@ -371,11 +378,37 @@ static PetscErrorCode PCSetFromOptions_Jacobi(PetscOptionItems *PetscOptionsObje
   if (flg) {
     ierr = PCJacobiSetType(pc,type);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsBool("-pc_jacobi_abs","Use absolute values of diagaonal entries","PCJacobiSetUseAbs",jac->useabs,&jac->useabs,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-pc_jacobi_abs","Use absolute values of diagonal entries","PCJacobiSetUseAbs",jac->useabs,&jac->useabs,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-pc_jacobi_fixdiagonal","Fix null terms on diagonal","PCJacobiSetFixDiagonal",jac->fixdiag,&jac->fixdiag,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PCView_Jacobi(PC pc, PetscViewer viewer)
+{
+  PC_Jacobi     *jac = (PC_Jacobi *) pc->data;
+  PetscBool      iascii;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject) viewer, PETSCVIEWERASCII, &iascii);CHKERRQ(ierr);
+  if (iascii) {
+    PCJacobiType      type;
+    PetscBool         useAbs,fixdiag;
+    PetscViewerFormat format;
+
+    ierr = PCJacobiGetType(pc, &type);CHKERRQ(ierr);
+    ierr = PCJacobiGetUseAbs(pc, &useAbs);CHKERRQ(ierr);
+    ierr = PCJacobiGetFixDiagonal(pc, &fixdiag);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "  type %s%s%s\n", PCJacobiTypes[type], useAbs ? ", using absolute value of entries" : "", !fixdiag ? ", not checking null diagonal entries" : "");CHKERRQ(ierr);
+    ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
+      ierr = VecView(jac->diag, viewer);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 /* -------------------------------------------------------------------------- */
 /*
    PCCreate_Jacobi - Creates a Jacobi preconditioner context, PC_Jacobi,
@@ -393,13 +426,13 @@ static PetscErrorCode PCSetFromOptions_Jacobi(PetscOptionItems *PetscOptionsObje
 
    Options Database Key:
 +    -pc_jacobi_type <diagonal,rowmax,rowsum> - approach for forming the preconditioner
--    -pc_jacobi_abs - use the absolute value of the diagonal entry
+.    -pc_jacobi_abs - use the absolute value of the diagonal entry
+-    -pc_jacobi_fixdiag - fix for zero diagonal terms
 
    Level: beginner
 
-  Concepts: Jacobi, diagonal scaling, preconditioners
-
-  Notes: By using KSPSetPCSide(ksp,PC_SYMMETRIC) or -ksp_pc_side symmetric
+  Notes:
+    By using KSPSetPCSide(ksp,PC_SYMMETRIC) or -ksp_pc_side symmetric
          can scale each side of the matrix by the square root of the diagonal entries.
 
          Zero entries along the diagonal are replaced with the value 1.0
@@ -407,11 +440,10 @@ static PetscErrorCode PCSetFromOptions_Jacobi(PetscOptionItems *PetscOptionsObje
          See PCPBJACOBI for a point-block Jacobi preconditioner
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
-           PCJacobiSetType(), PCJacobiSetUseAbs(), PCJacobiGetUseAbs(), PCPBJACOBI
+           PCJacobiSetType(), PCJacobiSetUseAbs(), PCJacobiGetUseAbs(),
+           PCJacobiSetFixDiagonal(), PCJacobiGetFixDiagonal(), PCPBJACOBI
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_Jacobi"
 PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC pc)
 {
   PC_Jacobi      *jac;
@@ -429,11 +461,12 @@ PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC pc)
      Initialize the pointers to vectors to ZERO; these will be used to store
      diagonal entries of the matrix for fast preconditioner application.
   */
-  jac->diag      = 0;
-  jac->diagsqrt  = 0;
+  jac->diag      = NULL;
+  jac->diagsqrt  = NULL;
   jac->userowmax = PETSC_FALSE;
   jac->userowsum = PETSC_FALSE;
   jac->useabs    = PETSC_FALSE;
+  jac->fixdiag   = PETSC_TRUE;
 
   /*
       Set the pointers for the functions that are provided above.
@@ -448,8 +481,8 @@ PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC pc)
   pc->ops->reset               = PCReset_Jacobi;
   pc->ops->destroy             = PCDestroy_Jacobi;
   pc->ops->setfromoptions      = PCSetFromOptions_Jacobi;
-  pc->ops->view                = 0;
-  pc->ops->applyrichardson     = 0;
+  pc->ops->view                = PCView_Jacobi;
+  pc->ops->applyrichardson     = NULL;
   pc->ops->applysymmetricleft  = PCApplySymmetricLeftOrRight_Jacobi;
   pc->ops->applysymmetricright = PCApplySymmetricLeftOrRight_Jacobi;
 
@@ -457,14 +490,14 @@ PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC pc)
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetType_C",PCJacobiGetType_Jacobi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiSetUseAbs_C",PCJacobiSetUseAbs_Jacobi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetUseAbs_C",PCJacobiGetUseAbs_Jacobi);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiSetFixDiagonal_C",PCJacobiSetFixDiagonal_Jacobi);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCJacobiGetFixDiagonal_C",PCJacobiGetFixDiagonal_Jacobi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiSetUseAbs"
 /*@
    PCJacobiSetUseAbs - Causes the Jacobi preconditioner to use the
-      absolute values of the digonal divisors in the preconditioner
+      absolute values of the diagonal divisors in the preconditioner
 
    Logically Collective on PC
 
@@ -475,12 +508,11 @@ PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC pc)
    Options Database Key:
 .  -pc_jacobi_abs
 
-   Notes: This takes affect at the next construction of the preconditioner
+   Notes:
+    This takes affect at the next construction of the preconditioner
 
    Level: intermediate
 
-   Concepts: Jacobi preconditioner
-
 .seealso: PCJacobiaSetType(), PCJacobiGetUseAbs()
 
 @*/
@@ -494,11 +526,9 @@ PetscErrorCode  PCJacobiSetUseAbs(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiGetUseAbs"
 /*@
    PCJacobiGetUseAbs - Determines if the Jacobi preconditioner uses the
-      absolute values of the digonal divisors in the preconditioner
+      absolute values of the diagonal divisors in the preconditioner
 
    Logically Collective on PC
 
@@ -513,8 +543,6 @@ PetscErrorCode  PCJacobiSetUseAbs(PC pc,PetscBool flg)
 
    Level: intermediate
 
-   Concepts: Jacobi preconditioner
-
 .seealso: PCJacobiaSetType(), PCJacobiSetUseAbs(), PCJacobiGetType()
 
 @*/
@@ -528,8 +556,65 @@ PetscErrorCode  PCJacobiGetUseAbs(PC pc,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiSetType"
+/*@
+   PCJacobiSetFixDiagonal - Do not check for zero values on diagonal
+
+   Logically Collective on PC
+
+   Input Parameters:
++  pc - the preconditioner context
+-  flg - the boolean flag
+
+   Options Database Key:
+.  -pc_jacobi_fixdiagonal
+
+   Notes:
+    This takes affect at the next construction of the preconditioner
+
+   Level: intermediate
+
+.seealso: PCJacobiSetType(), PCJacobiGetFixDiagonal()
+
+@*/
+PetscErrorCode  PCJacobiSetFixDiagonal(PC pc,PetscBool flg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCJacobiSetFixDiagonal_C",(PC,PetscBool),(pc,flg));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PCJacobiGetFixDiagonal - Determines if the Jacobi preconditioner checks for zero diagonal terms
+
+   Logically Collective on PC
+
+   Input Parameter:
+.  pc - the preconditioner context
+
+   Output Parameter:
+.  flg - the boolean flag
+
+   Options Database Key:
+.  -pc_jacobi_fixdiagonal
+
+   Level: intermediate
+
+.seealso: PCJacobiSetType(), PCJacobiSetFixDiagonal()
+
+@*/
+PetscErrorCode  PCJacobiGetFixDiagonal(PC pc,PetscBool *flg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscUseMethod(pc,"PCJacobiGetFixDiagonal_C",(PC,PetscBool*),(pc,flg));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    PCJacobiSetType - Causes the Jacobi preconditioner to use either the diagonal, the maximum entry in each row,
       of the sum of rows entries for the diagonal preconditioner
@@ -545,8 +630,6 @@ PetscErrorCode  PCJacobiGetUseAbs(PC pc,PetscBool *flg)
 
    Level: intermediate
 
-   Concepts: Jacobi preconditioner
-
 .seealso: PCJacobiaUseAbs(), PCJacobiGetType()
 @*/
 PetscErrorCode  PCJacobiSetType(PC pc,PCJacobiType type)
@@ -559,8 +642,6 @@ PetscErrorCode  PCJacobiSetType(PC pc,PCJacobiType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCJacobiGetType"
 /*@
    PCJacobiGetType - Gets how the diagonal matrix is produced for the preconditioner
 
@@ -574,8 +655,6 @@ PetscErrorCode  PCJacobiSetType(PC pc,PCJacobiType type)
 
    Level: intermediate
 
-   Concepts: Jacobi preconditioner
-
 .seealso: PCJacobiaUseAbs(), PCJacobiSetType()
 @*/
 PetscErrorCode  PCJacobiGetType(PC pc,PCJacobiType *type)
