@@ -36,9 +36,6 @@ typedef struct {
   PetscScalar       *lvec0, *lvec1;
 } PC_PARMS;
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_PARMS"
 static PetscErrorCode PCSetUp_PARMS(PC pc)
 {
   Mat               pmat;
@@ -105,22 +102,22 @@ static PetscErrorCode PCSetUp_PARMS(PC pc)
     if (ia[i-low+1] >= length) {
       length += ncols;
       ierr    = PetscMalloc1(length,&ja1);CHKERRQ(ierr);
-      ierr    = PetscMemcpy(ja1,ja,(ia[i-low]-1)*sizeof(int));CHKERRQ(ierr);
+      ierr    = PetscArraycpy(ja1,ja,ia[i-low]-1);CHKERRQ(ierr);
       ierr    = PetscFree(ja);CHKERRQ(ierr);
       ja      = ja1;
       ierr    = PetscMalloc1(length,&aa1);CHKERRQ(ierr);
-      ierr    = PetscMemcpy(aa1,aa,(ia[i-low]-1)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr    = PetscArraycpy(aa1,aa,ia[i-low]-1);CHKERRQ(ierr);
       ierr    = PetscFree(aa);CHKERRQ(ierr);
       aa      = aa1;
     }
-    ierr = PetscMemcpy(&ja[pos],cols,ncols*sizeof(int));CHKERRQ(ierr);
-    ierr = PetscMemcpy(&aa[pos],values,ncols*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(&ja[pos],cols,ncols);CHKERRQ(ierr);
+    ierr = PetscArraycpy(&aa[pos],values,ncols);CHKERRQ(ierr);
     ierr = MatRestoreRow(pmat,i,&ncols,&cols,&values);CHKERRQ(ierr);
   }
 
   /* csr info is for local matrix so initialize im[] locally */
   ierr = PetscMalloc1(lsize,&im);CHKERRQ(ierr);
-  ierr = PetscMemcpy(im,&maptmp[mapptr[rank]-1],lsize*sizeof(int));CHKERRQ(ierr);
+  ierr = PetscArraycpy(im,&maptmp[mapptr[rank]-1],lsize);CHKERRQ(ierr);
 
   /* 1-based indexing */
   for (i=0; i<ia[lsize]-1; i++) ja[i] = ja[i]+1;
@@ -183,8 +180,6 @@ static PetscErrorCode PCSetUp_PARMS(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_PARMS"
 static PetscErrorCode PCView_PARMS(PC pc,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -245,8 +240,6 @@ static PetscErrorCode PCView_PARMS(PC pc,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_PARMS"
 static PetscErrorCode PCDestroy_PARMS(PC pc)
 {
   PC_PARMS       *parms = (PC_PARMS*)pc->data;
@@ -274,8 +267,6 @@ static PetscErrorCode PCDestroy_PARMS(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_PARMS"
 static PetscErrorCode PCSetFromOptions_PARMS(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_PARMS          *parms = (PC_PARMS*)pc->data;
@@ -319,8 +310,6 @@ static PetscErrorCode PCSetFromOptions_PARMS(PetscOptionItems *PetscOptionsObjec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_PARMS"
 static PetscErrorCode PCApply_PARMS(PC pc,Vec b,Vec x)
 {
   PetscErrorCode    ierr;
@@ -339,8 +328,6 @@ static PetscErrorCode PCApply_PARMS(PC pc,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetGlobal_PARMS"
 static PetscErrorCode PCPARMSSetGlobal_PARMS(PC pc,PCPARMSGlobalType type)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -353,8 +340,6 @@ static PetscErrorCode PCPARMSSetGlobal_PARMS(PC pc,PCPARMSGlobalType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetGlobal"
 /*@
    PCPARMSSetGlobal - Sets the global preconditioner to be used in PARMS.
 
@@ -390,8 +375,6 @@ PetscErrorCode PCPARMSSetGlobal(PC pc,PCPARMSGlobalType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetLocal_PARMS"
 static PetscErrorCode PCPARMSSetLocal_PARMS(PC pc,PCPARMSLocalType type)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -404,8 +387,6 @@ static PetscErrorCode PCPARMSSetLocal_PARMS(PC pc,PCPARMSLocalType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetLocal"
 /*@
    PCPARMSSetLocal - Sets the local preconditioner to be used in PARMS.
 
@@ -446,8 +427,6 @@ PetscErrorCode PCPARMSSetLocal(PC pc,PCPARMSLocalType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetSolveTolerances_PARMS"
 static PetscErrorCode PCPARMSSetSolveTolerances_PARMS(PC pc,PetscReal tol,PetscInt maxits)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -464,8 +443,6 @@ static PetscErrorCode PCPARMSSetSolveTolerances_PARMS(PC pc,PetscReal tol,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetSolveTolerances"
 /*@
    PCPARMSSetSolveTolerances - Sets the convergence tolerance and the maximum iterations for the
    inner GMRES solver, when the Schur global preconditioner is used.
@@ -498,8 +475,6 @@ PetscErrorCode PCPARMSSetSolveTolerances(PC pc,PetscReal tol,PetscInt maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetSolveRestart_PARMS"
 static PetscErrorCode PCPARMSSetSolveRestart_PARMS(PC pc,PetscInt restart)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -512,8 +487,6 @@ static PetscErrorCode PCPARMSSetSolveRestart_PARMS(PC pc,PetscInt restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetSolveRestart"
 /*@
    PCPARMSSetSolveRestart - Sets the number of iterations at which the
    inner GMRES solver restarts.
@@ -544,8 +517,6 @@ PetscErrorCode PCPARMSSetSolveRestart(PC pc,PetscInt restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetNonsymPerm_PARMS"
 static PetscErrorCode PCPARMSSetNonsymPerm_PARMS(PC pc,PetscBool nonsym)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -558,8 +529,6 @@ static PetscErrorCode PCPARMSSetNonsymPerm_PARMS(PC pc,PetscBool nonsym)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetNonsymPerm"
 /*@
    PCPARMSSetNonsymPerm - Sets the type of permutation for the ARMS preconditioner: the standard
    symmetric ARMS or the non-symmetric ARMS (ARMS-ddPQ).
@@ -591,8 +560,6 @@ PetscErrorCode PCPARMSSetNonsymPerm(PC pc,PetscBool nonsym)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetFill_PARMS"
 static PetscErrorCode PCPARMSSetFill_PARMS(PC pc,PetscInt lfil0,PetscInt lfil1,PetscInt lfil2)
 {
   PC_PARMS *parms = (PC_PARMS*)pc->data;
@@ -613,8 +580,6 @@ static PetscErrorCode PCPARMSSetFill_PARMS(PC pc,PetscInt lfil0,PetscInt lfil1,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPARMSSetFill"
 /*@
    PCPARMSSetFill - Sets the fill-in parameters for ILUT, ILUK and ARMS preconditioners.
    Consider the original matrix A = [B F; E C] and the approximate version
@@ -682,15 +647,13 @@ PetscErrorCode PCPARMSSetFill(PC pc,PetscInt lfil0,PetscInt lfil1,PetscInt lfil2
    IMPORTANT:
    Unless configured appropriately, this preconditioner performs an inexact solve
    as part of the preconditioner application. Therefore, it must be used in combination
-   with flexible variants of iterative solvers, such as KSPFGMRES or KSPCGR.
+   with flexible variants of iterative solvers, such as KSPFGMRES or KSPGCR.
 
    Level: intermediate
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_PARMS"
 PETSC_EXTERN PetscErrorCode PCCreate_PARMS(PC pc)
 {
   PC_PARMS       *parms;
